package infomat.algorithms.description;

import infomat.vectorspace.*;
import infomat.vectorspace.similarity.*;
import infomat.vectorspace.ivector.*;

import mro.util.*;
import mro.util.properties.*;

import java.util.*;


/**
 * Constructs ILabels for IObjectGroups.
 *
 * @version          090104
 * @author           Magnus Rosell
 *
 */
public class CentroidRankingBoostDescriber extends Describer{


    public mro.util.properties.Properties getDefaultProperties(){

	mro.util.properties.Properties properties = 
	    new mro.util.properties.Properties("Centroid Ranking Boost Describer Properties", CurrentClassGetter.getClassName());
	
	    /*			
	try{
	    
			Property p = new Property("Number of clusters", 5, "Number of clusters");
	    properties.addProperty(p);
	
	}catch(PropertyNameConflictException pnce){
	    // Won't happen here
	}

	    */

	return properties;
    }


    public CentroidRankingBoostDescriber(){
	super();
    }

    /*
    private IVector<IObject> describeGroup(IVector<IObject> firstV, 
					   IVector<IObject> secondV, 
					   SparseISimilarity similarity){
	IVector<IObject> newFV = new IVector<IObject>(firstV);
	IVector<IObject> newSV = new IVector<IObject>(secondV);
	newFV.log(2);
	newSV.log(2);
	
	return newFV.subtract(newSV);
    }
    */

    private IVector<IVectorIndex> rankLog(IVector<IVectorIndex> ivec){
	IVector<IVectorIndex> ret = new IVector<IVectorIndex>();
	Vector<IVectorIndex> vec = ivec.getOrderedVector();
	for(int i=0; i<vec.size(); i++){
	    IVectorIndex ii = vec.get(i);
	    ret.put(ii, (float) MoreMath.log(vec.size()-i, 2)); 
	}
	return ret;
    }



    private static <T extends IVectorIndex> IVector<T> subtract(IVector<T> termLarge, IVector<T> termSmall){
	IVector<T> retVec = new IVector<T>();
	
	Vector<T> vecSmall = termSmall.getUnorderedVector();
	for(int i=0; i<vecSmall.size(); i++){
	    T tSmall = vecSmall.get(i);
	    Float fLarge = termLarge.getValue(tSmall);
	    if(fLarge!=null){
		retVec.put(tSmall, fLarge - termSmall.getValue(tSmall));
	    }
	}

	return retVec;
    }







    /**
     * Only works for flat clusterings yet.
     */
    public HashMap<IObjectGroup, IVector<IVectorIndex>> 
	describeGrouping(IObjectGrouping ioging, SparseISimilarity similarity){

	HashMap<IObjectGroup, IVector<IVectorIndex>> groupDescriptionMap = 
	    new HashMap<IObjectGroup, IVector<IVectorIndex>>();

	IVector<IVectorIndex> parent = similarity.getGroupRepresentation(ioging);
	parent = rankLog(parent);

	Vector<IObjectGroup> groups = ioging.getGroups();
	for(int i=0; i<groups.size(); i++){
	    IObjectGroup group = groups.get(i);
	    IVector<IVectorIndex> child = similarity.getGroupRepresentation(group);
	    child=rankLog(child);
	    
	    //	    IVector<IVectorIndex> parentCopy = new IVector<IVectorIndex>(parent);
	    //	    parentCopy.subtract(child);
	    //	    groupDescriptionMap.put(group, parentCopy);
	    
	    IVector<IVectorIndex> ivec = subtract(parent, child);
	    groupDescriptionMap.put(group, ivec);
	}
	
	return groupDescriptionMap;

	/*

	HierarchicalIObjectGrouping hioging = null;

	if(ioging instanceof HierarchicalIObjectGrouping){
	    hioging = (HierarchicalIObjectGrouping) ioging;

	}else{
	    hioging = new HierarchicalIObjectGrouping("Temporary HierarchicalIObjectGrouping");
	    IObjectGroupNode root = new IObjectGroupNode(ioging, null);
	    Vector<IObjectGroupNode> nodeVec = new Vector<IObjectGroupNode>();
	    nodeVec.add(root);
	    hioging.setRootIObjectGroupNodes(nodeVec);

	    nodeVec = new Vector<IObjectGroupNode>();
	    Vector<IObjectGroup> groups = ioging.getGroups();
	    IObjectGroupNode parent = new IObjectGroupNode(ioging, null);
	    for(int i=0; i<groups.size(); i++){
		IObjectGroup iog = groups.elementAt(i);
		IObjectGroupNode node = new IObjectGroupNode(iog, parent);
		nodeVec.add(node);
	    }
	    hioging.setBottomIObjectGroupNodes(nodeVec);
	}

	HashMap<IObjectGroup, ILabel> groupLabelMap = 
	    new HashMap<IObjectGroup, ILabel>();
	    
	HashMap<IObjectGroupNode, HashMap<IObject, Float>> parentMapMap =
	    new HashMap<IObjectGroupNode, HashMap<IObject, Float>>();

	Vector<IObjectGroupNode> bottomNodes = 
	    hioging.getBottomIObjectGroupNodes();

	for(int i=0; i<bottomNodes.size(); i++){
	    IObjectGroupNode node = bottomNodes.elementAt(i);
	    IObjectGroupNode parent = node.getParent();

	    HashMap<IObject, Float> objWeightOrderMap = 
		parentMapMap.get(parent);
	    if(objWeightOrderMap==null){
		Vector<Map.Entry<IObject, Float>> objWeightVec = 
		    getSortedGroupRepresentation(parent.getIObjectGroup(), similarity);
		objWeightOrderMap = new HashMap<IObject, Float>();
		int rank=1;
		for(int j=0; j<objWeightVec.size(); j++){
		    IObject io = objWeightVec.elementAt(j).getKey();
		    objWeightOrderMap.put(io, (float) rank);
		    rank++;
		}
		parentMapMap.put(parent, objWeightOrderMap);
	    }

	    IObjectGroup iog = node.getIObjectGroup();
	    ILabel label = 
		labelGroup(iog, similarity, objWeightOrderMap);
	    groupLabelMap.put(iog, label);
	}

	return groupLabelMap;
	*/
    }

    public String getName(){
	return "Centroid Ranking Boost Describer";
    }


    public Vector getPropertyVector(){
    	return properties.getPropertyVector();
    }
    

    public Object getPropertyValue(String name){
    	return properties.getPropertyValue(name);
    }
    public void setProperty(String name, Object value) throws PropertyNotFoundException{
    	properties.setProperty(name, value);
    }

    public mro.util.properties.Properties getProperties(){
    	return properties;
    }
    public void setProperties(mro.util.properties.Properties properties){
    	this.properties=properties;
    }



}

