package infomat.algorithms.description;

import infomat.vectorspace.*;
import infomat.vectorspace.similarity.*;
import infomat.vectorspace.ivector.*;

import mro.util.*;
import mro.util.properties.*;

import java.util.*;


/**
 * Constructs ILabels for IObjectGroups.
 *
 * @version          090104
 * @author           Magnus Rosell
 *
 */
public class CombinedCentroidEntropyDescriber extends Describer{

    public CombinedCentroidEntropyDescriber(){
	this.properties = getDefaultProperties();
    }

    public CombinedCentroidEntropyDescriber(mro.util.properties.Properties properties){
	this.properties = properties;
    }

    public mro.util.properties.Properties getDefaultProperties(){

	mro.util.properties.Properties properties = new mro.util.properties.Properties("Combined Centroid Entropy Describer Properties", CurrentClassGetter.getClassName());
		
	    /*
	try{

	    Property p = new Property("Number of objects in labels", 10, "The number of objects in each label");
	    properties.addProperty(p);

	}catch(PropertyNameConflictException pnce){
	    // Won't happen here
	}
	    */

	return properties;
    }


    public Object getPropertyValue(String name){
	return properties.getPropertyValue(name);
    }

    public void setProperty(String name, Object value) 
	throws PropertyNotFoundException{
	properties.setProperty(name, value);
    }

    public mro.util.properties.Properties getProperties(){
	return properties;
    }

    public void setProperties(mro.util.properties.Properties properties){
	this.properties=properties;
    }



    /*********************************/


    /**
     * Returns null if CentroidDescriber 
     * and CentroidEntropyDescriber do not match!
     */
    public HashMap<IObjectGroup, IVector<IVectorIndex>> describeGrouping(IObjectGrouping ioging, SparseISimilarity similarity){


	HashMap<IObjectGroup, IVector<IVectorIndex>> groupDescriptionMap = 
	    new HashMap<IObjectGroup, IVector<IVectorIndex>>();

	CentroidDescriber cd = new CentroidDescriber();
	HashMap<IObjectGroup, IVector<IVectorIndex>> cdGroupDescriptionMap = 
	    cd.describeGrouping(ioging, similarity);

	CentroidEntropyDescriber ceb = 
	    new CentroidEntropyDescriber();
	HashMap<IObjectGroup, IVector<IVectorIndex>> cebGroupDescriptionMap =
	    ceb.describeGrouping(ioging, similarity);
	

	Vector<IObjectGroup> groups = ioging.getGroups();
	for(int i=0; i<groups.size(); i++){
	    IObjectGroup iog = groups.elementAt(i);

	    IVector<IVectorIndex> cdVec = cdGroupDescriptionMap.get(iog);
	    IVector<IVectorIndex> cebVec = cebGroupDescriptionMap.get(iog);
	    
	    if(cdVec==null || cebVec==null){
		System.out.println("CombinedCentroidEntropyDescriber: CentroidDescriber and CentroidEntropyDescriber do not match!");
		return null;
	    }
	    
	    IVector<IVectorIndex> vec = IVector.dotMultiply(cdVec, cebVec);
	    groupDescriptionMap.put(iog, vec);
	}

	return groupDescriptionMap;
    }

    /*********************************/
   
    public String getName(){
	return "Combined Centroid Entropy Describer";
    }



}

