package infomat.algorithms.description;

import infomat.vectorspace.*;
import infomat.vectorspace.similarity.*;
import infomat.vectorspace.ivector.*;

import mro.util.*;
import mro.util.properties.*;

import java.util.*;


/**
 *
 *
 * @version          090115
 * @author           Magnus Rosell
 *
 */
public class DescriptionManipulator{



    /*********************************/

    /**
     * Returns a description with the same size as the one given.
     * For each IVectorIndex in each group the value is multiplied
     * by the overall entropy of it.
     */
    public static HashMap<IObjectGroup, IVector<IVectorIndex>> multiplyIndexesByEntropy(HashMap<IObjectGroup, IVector<IVectorIndex>> inGroupDescriptionMap){

	HashMap<IObjectGroup, IVector<IVectorIndex>> retGroupDescriptionMap
	    = new HashMap<IObjectGroup, IVector<IVectorIndex>>();


	Vector<IObjectGroup> groups = new Vector<IObjectGroup>();
	Iterator<IObjectGroup> itGroup = 
	    inGroupDescriptionMap.keySet().iterator();
	while(itGroup.hasNext()){
	    IObjectGroup iog = itGroup.next();
	    groups.add(iog);
	    retGroupDescriptionMap.put(iog, new IVector<IVectorIndex>());
	}

	IVector<IVectorIndex> allivec = new IVector<IVectorIndex>();
	Iterator<IVector<IVectorIndex>> it = 
	    inGroupDescriptionMap.values().iterator();
	while(it.hasNext()){
	    IVector<IVectorIndex> thisivec = it.next();
	    allivec.put(thisivec);
	}

	Vector<IVectorIndex> vec = allivec.getUnorderedVector();
	for(int i=0; i<vec.size(); i++){
	    IVectorIndex ii = vec.get(i);

	    double[] ps = new double[groups.size()];
	    double psum =0;
		
	    for(int g=0; g<groups.size(); g++){
		IObjectGroup iog = groups.elementAt(g);
		IVector<IVectorIndex> ivec = 
		    inGroupDescriptionMap.get(iog);
		Float f = ivec.getValue(ii);
		if(f!=null){
		    ps[g] = f; 
		    psum += f;
		}
	    }
		
	    double entropy = 0;
	    for(int g = 0; g<ps.length; g++){
		if(ps[g]!=0){
		    ps[g] /= psum;
		    entropy += - ps[g] * MoreMath.log(ps[g], 2);
		}
	    }
	    entropy = MoreMath.log(ps.length, 2) - entropy; // Info Gain


	    for(int g=0; g<groups.size(); g++){
		IObjectGroup iog = groups.elementAt(g);
		IVector<IVectorIndex> inVec = 
		    inGroupDescriptionMap.get(iog);
		if(inVec==null)
		    continue;
		IVector<IVectorIndex> retVec = 
		    retGroupDescriptionMap.get(iog);
		/*
		if(retVec==null){
		    retVec = new IVector<IVectorIndex>();
		    retGroupDescriptionMap.put(iog, retVec);
		}
		*/

		Float f = inVec.getValue(ii);
		if(f!=null){
		    retVec.put(ii, (float) (f*entropy));
		}
	    }
	}

	return retGroupDescriptionMap;
    }


    public static HashMap<IObjectGroup, IVector<IVectorIndex>> getTopDescription(HashMap<IObjectGroup, IVector<IVectorIndex>> clusteringDescription, int numberOfObjectsPerGroup){
	
	HashMap<IObjectGroup, IVector<IVectorIndex>> ret = 
	    new HashMap<IObjectGroup, IVector<IVectorIndex>>();

	
	Iterator<Map.Entry<IObjectGroup, IVector<IVectorIndex>>> it = clusteringDescription.entrySet().iterator();
	while(it.hasNext()){
	    Map.Entry<IObjectGroup, IVector<IVectorIndex>> entry = it.next();

	    IVector<IVectorIndex> newIvec = entry.getValue().highestRanked(numberOfObjectsPerGroup);
	    ret.put(entry.getKey(), newIvec);
	}

	return ret;
    }


    public static void printClusteringDescription(HashMap<IObjectGroup, IVector<IVectorIndex>> clusteringDescription, int numberPerGroup){
	
	
	Iterator<Map.Entry<IObjectGroup, IVector<IVectorIndex>>> it = clusteringDescription.entrySet().iterator();
	while(it.hasNext()){
	    Map.Entry<IObjectGroup, IVector<IVectorIndex>> entry = it.next();
	    
	    System.out.println(entry.getKey().getName());

	    IVector<IVectorIndex> ivec = entry.getValue();
	    Vector<IVectorIndex> vec = ivec.getOrderedVector();
	    
	    //	    int max = 5;
	    for(int i=0; i<vec.size() && (numberPerGroup >0 && i<numberPerGroup); i++){
		IVectorIndex ii = vec.get(i);
		System.out.println("  - "+ii.getString()+": "+ivec.getValue(ii));
	    }

	}


    }



}

