package infomat.algorithms.description;

import infomat.vectorspace.*;
import infomat.vectorspace.similarity.*;
import infomat.vectorspace.ivector.*;

import mro.util.*;
import mro.util.properties.*;

import java.util.*;


/**
 * Constructs ILabels for IObjectGroups.
 *
 * @version          090812
 * @author           Magnus Rosell
 *
 */
public class PredictiveDescriber extends Describer{

    public PredictiveDescriber(){
	this.properties = getDefaultProperties();
    }

    public PredictiveDescriber(mro.util.properties.Properties properties){
	this.properties = properties;
    }

    public mro.util.properties.Properties getDefaultProperties(){

	mro.util.properties.Properties properties = new mro.util.properties.Properties("Predictive Describer Properties", CurrentClassGetter.getClassName());
		
	    /*
	try{

	    Property p = new Property("Number of objects in labels", 10, "The number of objects in each label");
	    properties.addProperty(p);

	}catch(PropertyNameConflictException pnce){
	    // Won't happen here
	}
	    */

	return properties;
    }


    public Object getPropertyValue(String name){
	return properties.getPropertyValue(name);
    }

    public void setProperty(String name, Object value) 
	throws PropertyNotFoundException{
	properties.setProperty(name, value);
    }

    public mro.util.properties.Properties getProperties(){
	return properties;
    }

    public void setProperties(mro.util.properties.Properties properties){
	this.properties=properties;
    }



    /*********************************/


    /*
    private IVector<IVectorIndex> describeGroup(IObjectGroup iog, 
					  SparseISimilarity similarity){

	return similarity.getGroupRepresentation(iog);
    }
    */

    public HashMap<IObjectGroup, IVector<IVectorIndex>> describeGrouping(IObjectGrouping ioging, SparseISimilarity similarity){

	if(!(similarity instanceof SparseDotSimilarity))
	    return null;
	
	SparseDotSimilarity sim = (SparseDotSimilarity) similarity;
	IMatrix matrix = sim.getIMatrix();
	boolean row = sim.getIMatrixDimension();
	
	IVector<IVectorIndex> groupingFreqs = 
	    frequenciesInGroup(ioging, matrix, row);


	HashMap<IObjectGroup, IVector<IVectorIndex>> groupDescriptionMap = 
	    new HashMap<IObjectGroup, IVector<IVectorIndex>>();

	Vector<IObjectGroup> groups = ioging.getGroups();
	for(int i=0; i<groups.size(); i++){
	    IObjectGroup iog = groups.elementAt(i);

	    IVector<IVectorIndex> groupFreqs = 
		frequenciesInGroup(iog, matrix, row);
	    Vector<IVectorIndex> vec = groupFreqs.getUnorderedVector();
	    for(int j=0; j<vec.size(); j++){
		IVectorIndex ii = vec.get(j);
		Float groupF = groupFreqs.getValue(ii);
		Float groupingF = groupingFreqs.getValue(ii);
		if(groupF!=null && groupingF!=null)
		    groupFreqs.put(ii, groupF/groupingF);
		else
		    groupFreqs.put(ii, 0.0f);
	    }
	    
	    groupDescriptionMap.put(iog, groupFreqs);
	}

	return groupDescriptionMap;
    }

    private IVector<IVectorIndex> frequenciesInGroup(IObjectGroup group, IMatrix matrix, boolean row){
	IVector<IVectorIndex> ret = new IVector<IVectorIndex>();
	
	Vector<IObject> ios = group.getOrdered();
	Float totalCount = 0.0f;
	HashSet set = null;
	for(int i=0; i<ios.size(); i++){
	    IObject io = ios.get(i);
	    if(row)
		set = matrix.getColumnIMatrixCellsForRowObject(io);
	    else
		set = matrix.getRowIMatrixCellsForColumnObject(io);

	    Iterator it = set.iterator();
	    while(it.hasNext()){
		IMatrixCell mc = (IMatrixCell) it.next();
		IObject io2 = mc.getOtherIObject(io);
		float localCount = mc.getLocalCount();
		totalCount += localCount;
		Float f = ret.getValue(io2);
		if(f==null)
		    ret.put(io2, localCount);
		else
		    ret.put(io2, f+localCount);
	    }
	}

	Vector<IVectorIndex> vec = ret.getUnorderedVector();
	for(int i=0; i<vec.size(); i++){
	    IVectorIndex ii = vec.get(i);
	    Float f = ret.getValue(ii);
	    ret.put(ii, f/totalCount);
	}

	return ret;
    }


    /*********************************/
   
    public String getName(){
	return "Predictive Describer";
    }



}

