package infomat.algorithms.grouping;

import infomat.vectorspace.*;
import infomat.vectorspace.similarity.*;
import infomat.vectorspace.ivector.*;

import java.util.*;

import mro.util.CurrentClassGetter;
import mro.util.UniqueIdentifier;
import mro.util.properties.Property;
import mro.util.properties.PropertyNameConflictException;
import mro.util.properties.PropertyNotFoundException;
import mro.util.properties.PropertyValueDescription;

/**
  * The other IObjectGrouping should be along the other dimension.
 *
 */
public class AppearanceRelativeClusterer extends GroupingAlgorithm 
    implements NeedsIMatrix, NeedsIMatrixDimension, NeedsOtherIObjectGrouping{

    private static int relativeClusteringNumber=1;
	
    public mro.util.properties.Properties getDefaultProperties(){

	mro.util.properties.Properties properties = new 
	    mro.util.properties.Properties("Appearance RelativeClusterer Properties", CurrentClassGetter.getClassName());

	try{

	    Property p = new Property("Type", 0, "What version.");
	    PropertyValueDescription pvd = new PropertyValueDescription("0", "As many clusters as in the relative grouping.");
	    p.addValueDescription(pvd);
	    pvd = new PropertyValueDescription("1", "One extra cluster with all objects that appear in more than \"Number of Groups\" number of groups.");
	    p.addValueDescription(pvd);
	    pvd = new PropertyValueDescription("2", "Just one cluster.");
	    p.addValueDescription(pvd);
	    properties.addProperty(p);

	    p = new Property("Number Of Groups", 2, "Used in Type 1.");
	    properties.addProperty(p);

	}catch(PropertyNameConflictException pnce){
	    // Won't happen here
	}


	return properties;

    }

    private IMatrix matrix;
    private boolean row;
    IObjectGrouping otherIObjectGrouping;

    public AppearanceRelativeClusterer(){
	this.properties = getDefaultProperties();
    }

    /**
     * The algorithm clusters the opposite dimension specified
     * in the ISimilarity.
     */
    public IObjectGrouping cluster(IObjectGroup iogroup){


	//	IMatrix matrix = similarity.getIMatrix();
	IMatrix matrix = getIMatrix();
	IObjectSet otherSet = matrix.getRowIObjectSet();
	//	boolean row = similarity.getIMatrixDimension();
	boolean row = getIMatrixDimension();
	if(row)
	    otherSet = matrix.getColumnIObjectSet();

	IObjectGrouping otherGrouping = otherIObjectGrouping;

	IObjectGrouping relGrouping = 
	    makeWeightGroupingAppearanceRelativeGroupingOfOtherSet(iogroup, otherGrouping);
		
	String name = "Appearance Grouping relative ("+otherGrouping.getName()+") "+relativeClusteringNumber;
	relativeClusteringNumber++;		
	relGrouping.setName(name);

	return relGrouping;

    }




    private IObjectGrouping 
	makeWeightGroupingAppearanceRelativeGroupingOfOtherSet(IObjectGroup iogroup, IObjectGrouping grouping){

	IObjectGrouping retGrouping = null;

	int type = (Integer) properties.getPropertyValue("Type");

	if(type==0 || type == 2)
	    retGrouping = constructRelativeGrouping(iogroup, grouping, type, false);
	else if(type==1)
	    retGrouping = constructRelativeGroupingWithExtraCommonGroup(iogroup, grouping);
	return retGrouping;
    }







    private IObjectGrouping 
	constructRelativeGrouping(IObjectGroup iogroup, IObjectGrouping grouping, int type, boolean noSoFar){
		
	// Tillverka ny gruppering
	IObjectGrouping coging = 
	    new IObjectGrouping("Appearance Grouping relative "+grouping.getName());

	HashSet<IObject> soFarSet = new HashSet<IObject>();

	Vector<IObjectGroup> groups = null;
	if(type==0) // As many clusters
	    groups = grouping.getGroups();
	if(type==2){ // Just one cluster
	    groups = new Vector<IObjectGroup>();
	    groups.add(grouping);
	}

	for(int i=0; i<groups.size(); i++){
	    IObjectGroup cog = groups.elementAt(i);
	    IObjectGroup newCOG = new IObjectGroup("Appearance Group relative "+cog.getString());	    
	    

	    int number = 0;
	    Vector<IObject> cios = cog.getOrdered();
	    for(int j=0; j<cios.size(); j++){
		IObject cio = cios.get(j);
		HashSet cellSet = null;
		if(row) 
		    cellSet = matrix.getRowIMatrixCellsForColumnObject(cio);
		else
		    cellSet = matrix.getColumnIMatrixCellsForRowObject(cio);
		

		IVector<IObject> localVec = new IVector<IObject>();
		Iterator it = cellSet.iterator();
		while(it.hasNext()){
		    IMatrixCell mc = (IMatrixCell) it.next();
		    IObject io = null;
		    if(row)
			io = mc.getRowIObject();
		    else
			io = mc.getColumnIObject();
		    if(noSoFar || !soFarSet.contains(io)){
			localVec.add(io, (float) -io.getID());
			soFarSet.add(io);
		    }
		}
		Vector<IObject> vec = localVec.getOrderedVector();
		for(int k=0; k<vec.size(); k++){
		    IObject io = vec.get(k);
		    newCOG.add(io, number);
		    number++;		    
		}


		/*
		Iterator it = cellSet.iterator();
		while(it.hasNext()){
		    IMatrixCell mc = (IMatrixCell) it.next();
		    IObject io = null;
		    if(row)
			io = mc.getRowIObject();
		    else
			io = mc.getColumnIObject();
		    if(!soFarSet.contains(io)){
			newCOG.add(io, number);
			number++;
			soFarSet.add(io);
		    }
		}
		*/

	    }
	    newCOG.sortInCurrentVectorOrderWithInvertedOrderNumberAsValue();

	    if(newCOG.numberOfIObjects()>0)
		coging.add(newCOG);
	}

	coging.printGroupNamesAndSizes();

	return coging;
    }




    private IObjectGrouping 
	constructRelativeGroupingWithExtraCommonGroup(IObjectGroup iogroup, IObjectGrouping grouping){

//	Vector<IObjectGroup> newGroups = new Vector<IObjectGroup>();
		
// 	// Make groups with all words.
// 	Vector<IObjectGroup> groups = grouping.getGroups();
// 	for(int i=0; i<groups.size(); i++){
// 	    IObjectGroup cog = groups.elementAt(i);
// 	    IObjectGroup newCOG = new IObjectGroup("Appearance Group relative "+cog.getString());	    
	    
// 	    int number = 0;
// 	    HashSet<IObject> soFarSet = new HashSet<IObject>();
// 	    Vector<IObject> cios = cog.getOrdered();
// 	    for(int j=0; j<cios.size(); j++){
// 		IObject cio = cios.get(j);
// 		HashSet cellSet = null;
// 		if(row) 
// 		    cellSet = matrix.getRowIMatrixCellsForColumnObject(cio);
// 		else
// 		    cellSet = matrix.getColumnIMatrixCellsForRowObject(cio);
		
		
// 		IVector<IObject> localVec = new IVector<IObject>();
// 		Iterator it = cellSet.iterator();
// 		while(it.hasNext()){
// 		    IMatrixCell mc = (IMatrixCell) it.next();
// 		    IObject io = null;
// 		    if(row)
// 			io = mc.getRowIObject();
// 		    else
// 			io = mc.getColumnIObject();
// 		    if(!soFarSet.contains(io)){
// 			localVec.add(io, (float) -io.getID());
// 			soFarSet.add(io);
// 		    }
// 		}

// 		Vector<IObject> vec = localVec.getOrderedVector();
// 		for(int k=0; k<vec.size(); k++){
// 		    IObject io = vec.get(k);
// 		    newCOG.add(io, number);
// 		    number++;		    
// 		}
// 	    }

// 	    if(newCOG.numberOfIObjects()>0)
// 		newGroups.add(newCOG);
// 	}

	int type = 0; // "As many clusters as in the relative grouping."
	IObjectGrouping ordinaryRelative = 
	    constructRelativeGrouping(iogroup, grouping, type, true);

	IObjectGroup tmpGroup = new IObjectGroup(iogroup);
	System.out.println("tmpGroup: "+tmpGroup.numberOfIObjects());

	int nrClustersLimit = (Integer) getPropertyValue("Number Of Groups");

	int number = 0;
	Vector<IObjectGroup> ordinaryGroups = ordinaryRelative.getGroups();
	IVector<IObject> commonVec = new IVector<IObject>();
	Vector<IObject> ios = iogroup.getOrdered();
	for(int i=0; i<ios.size(); i++){
	    IObject io = ios.get(i);
	    int nr = 0;
	    boolean inAll = true;
	    for(int j=0; j<ordinaryGroups.size(); j++){
		IObjectGroup cog = ordinaryGroups.elementAt(j);
		if(cog.contains(io)){
		    nr++;
		}
	    }
	    if(nr>nrClustersLimit){
		//		commonVec.add(io, (float) -io.getID());
		commonVec.add(io, (float) number);

		tmpGroup.remove(io);
	    }
	}
	/*
	IObjectGroup tmpGroup = new IObjectGroup(iogroup);
	Vector<IObject> vec = commonVec.getOrderedVector();
	for(int i=0; i<vec.size(); i++){
	    IObject io = vec.get(i);
	    tmpGroup.remove(io);
	}
	*/

	System.out.println("tmpGroup: "+tmpGroup.numberOfIObjects());

	IObjectGrouping otherIObjectGrouping = getOtherIObjectGrouping();
	AppearanceRelativeClusterer arc = new AppearanceRelativeClusterer();
	arc.setIMatrixDimension(getIMatrixDimension());
	arc.setOtherIObjectGrouping(otherIObjectGrouping);
	IMatrix subMatrix = null;
	boolean row = getIMatrixDimension();
	if(row)
	    subMatrix = matrix.subMatrix(tmpGroup, otherIObjectGrouping);
	else
	    subMatrix = matrix.subMatrix(otherIObjectGrouping, tmpGroup);
	arc.setIMatrix(subMatrix);

	try{
	    arc.setProperty("Type", 0);
	}catch(PropertyNotFoundException pnfe){
	    // Not!!!
	}

	IObjectGrouping relative = arc.cluster(tmpGroup);

	Vector<IObjectGroup> groups = relative.getGroups();

	/*
	System.out.println("tmpGroup: "+tmpGroup.numberOfIObjects());
	IObjectGrouping relative = 
	    constructRelativeGrouping(tmpGroup, grouping, type, false);
	Vector<IObjectGroup> groups = relative.getGroups();
	*/

	/*
	Vector<IObject> vec = commonVec.getOrderedVector();
	for(int i=0; i<vec.size(); i++){
	    IObject io = vec.get(i);
	    for(int k=0; k<newGroups.size(); k++){
		IObjectGroup cog = newGroups.elementAt(k);
		cog.remove(io);
	    }
	}
	*/

	number = 0;
	IObjectGroup commonGroup = new IObjectGroup("Common Group");
	Vector<IObject> vec = commonVec.getOrderedVector();
	for(int i=0; i<vec.size(); i++){
	    IObject io = vec.get(i);
	    commonGroup.add(io, number);
	    number++;		    

	    /*
	    for(int j=0; j<groups.size(); j++){
		IObjectGroup g = groups.get(j);
		g.remove(io);
	    }
	    */

	}
	if(commonGroup.numberOfIObjects()>0){
	    commonGroup.sortInCurrentVectorOrderWithInvertedOrderNumberAsValue();
	    relative.add(commonGroup);
	}

	relative.removeEmptyGroups();
	relative.sort();

	relative.printGroupNamesAndSizes();

	return relative;

	/*
	// Tillverka ny gruppering
	IObjectGrouping coging = 
	    new IObjectGrouping("Appearance Grouping relative "+grouping.getName());
	if(commonGroup.numberOfIObjects()>0){
	    commonGroup.sortInCurrentVectorOrderWithInvertedOrderNumberAsValue();
	    coging.add(commonGroup);
	}
	for(int j=0; j<newGroups.size(); j++){
	    IObjectGroup cog = newGroups.elementAt(j);
	    cog.sortInCurrentVectorOrderWithInvertedOrderNumberAsValue();
	}



	coging.printGroupNamesAndSizes();

	return coging;
	*/
    }










    public String getInfoString(){
	return "Makes a clustering relative to the grouping of the other dimension.";
    }

    public String getAlgorithmName(){
	return "AppearanceRelativeClusterer";    	
    }

    public Vector getPropertyVector(){
	return properties.getPropertyVector();
    }


    public Object getPropertyValue(String name){
	return properties.getPropertyValue(name);
    }
    public void setProperty(String name, Object value) throws PropertyNotFoundException{
	properties.setProperty(name, value);
    }

    public mro.util.properties.Properties getProperties(){
	return properties;
    }
    public void setProperties(mro.util.properties.Properties properties){
	this.properties=properties;
    }

    public UniqueIdentifier getUniqueIdentifier(){
	return identifier;
    }

    public void setUniqueIdentifier(UniqueIdentifier uniqueIdentifier){
	this.identifier=uniqueIdentifier;
    }



    public void setIMatrix(IMatrix matrix){
	this.matrix=matrix;
    }

    public IMatrix getIMatrix(){
	return matrix;
    }

    public void setIMatrixDimension(boolean row){
	this.row=row;
    }

    public boolean getIMatrixDimension(){
	return this.row;
    }

    public void setOtherIObjectGrouping(IObjectGrouping grouping){
	this.otherIObjectGrouping = grouping;
    }

    private IObjectGrouping getOtherIObjectGrouping(){
	return otherIObjectGrouping;
    }


}
