package infomat.algorithms.search;

import infomat.algorithms.grouping.*;
import infomat.io.*;
import infomat.vectorspace.*;
import infomat.vectorspace.ivector.*;
import infomat.vectorspace.similarity.*;
import infomat.view.*;
import infomat.view.grouping.settings.*;
import infomat.view.util.*;
import infomat.view.vector.*;
import infomat.view.selector.*;

import mro.util.*;
import mro.util.properties.*;
import mro.util.properties.Properties;
import mro.util.properties.dialog.*;

import java.awt.*;
import java.awt.Dialog.*;//ModalityType;
import java.awt.event.*;
import javax.swing.border.*;
import java.util.*;
import java.io.*;

import java.util.regex.Pattern;

import javax.swing.*;
import javax.swing.filechooser.FileFilter;


	
/**
 * A search tool.
 *
 * @version          090729
 * @author           Magnus Rosell
 */
public class Searcher implements HasProperties, NeedsISimilarity{

    private mro.util.properties.Properties properties;
    private ISimilarity similarity;


    public Searcher(){
	this.properties = getDefaultProperties();
    }

    public Searcher(mro.util.properties.Properties properties){
	this.properties = properties;
    }

    public mro.util.properties.Properties getDefaultProperties(){

	mro.util.properties.Properties properties = new mro.util.properties.Properties("Searcher Properties", CurrentClassGetter.getClassName());
		
	try{

	    Property p = new Property("Least number of terms", 1, "The least number of terms that must be present for an object to be retrieved (must be >0)");

	    /*
	    PropertyValueDescription pvd = new PropertyValueDescription("0", "Each group reduced to \"Number objects\" in the current order.");
	    p.addValueDescription(pvd);
	    */

	    properties.addProperty(p);


	}catch(PropertyNameConflictException pnce){
	    // Won't happen here
	}

	return properties;
    }

    public Object getPropertyValue(String name){
	return properties.getPropertyValue(name);
    }

    public void setProperty(String name, Object value) 
	throws PropertyNotFoundException{
	properties.setProperty(name, value);
    }

    public mro.util.properties.Properties getProperties(){
	return properties;
    }

    public void setProperties(mro.util.properties.Properties properties){
	this.properties=properties;
    }



    public void setSimilarity(ISimilarity similarity){
	this.similarity = similarity;
    }

    public ISimilarity getSimilarity(){
	return this.similarity;
    }







    public IVector<IObject> search(IVector<IObject> queryVec, IMatrix matrix, IObjectGroup rowGroup, IObjectGroup columnGroup, boolean row){
	int number = (Integer) properties.getPropertyValue("Least number of terms");
	IVector<IObject> ret = basicSearch(queryVec, matrix, rowGroup, columnGroup, row, number);


	


	return ret;
    }


    /**
     * As basicSearch() but several different queries are considered.
     * When counting number of terms they must come from different
     * "queryVecs".
     */
    private IVector<IObject> multiSearch(Vector<IVector<IObject>> queryVecs, IMatrix matrix, IObjectGroup rowGroup, IObjectGroup columnGroup, boolean row, int number){
	if(queryVecs==null || queryVecs.size()==0)
	    return null;

	IObjectGroup objectGroup = rowGroup;
	IObjectGroup termGroup = columnGroup;
	if(!row){
	    objectGroup = columnGroup;
	    termGroup = rowGroup;
	}

	HashMap<IObject, HashSet<IObject>> termsForObjectMap = 
	    new HashMap<IObject, HashSet<IObject>>();
	Vector<IObject> resVec = new Vector<IObject>();

	for(int q=0; q<queryVecs.size(); q++){
	    IVector<IObject> queryVec = queryVecs.get(q);

	    Vector<IObject> cv = queryVec.getUnorderedVector();
	    if(cv.size()>0){
		IObject reprIO = cv.get(0);


		for(int i=0; i<cv.size(); i++){
		    IObject io = cv.get(i);
		    if(!termGroup.contains(io))
			continue;

		    HashSet cellSet = null;
		    if(row)
			cellSet = matrix.getRowIMatrixCellsForColumnObject(io);
		    else
			cellSet = matrix.getColumnIMatrixCellsForRowObject(io);

		    Vector cells = new Vector();
		    Iterator it = cellSet.iterator();
		    while(it.hasNext()){
			IMatrixCell mc = (IMatrixCell) it.next();
			IObject io2 = mc.getOtherIObject(io);
			if(objectGroup.contains(io2)){
			    resVec.add(io2);
			    HashSet<IObject> set = termsForObjectMap.get(io2);
			    if(set==null){
				set = new HashSet<IObject>();
				termsForObjectMap.put(io2, set);
			    }
			    set.add(reprIO);
			}
		    }
		}
	    }
	}

	resVec = removeObjectsWithFewTerms(resVec, termsForObjectMap, number);

	IVector<IObject> queryVec = new IVector<IObject>();
	for(int q=0; q<queryVecs.size(); q++)
	    queryVec.put(queryVecs.get(q));


	return rank(queryVec, resVec);
    }




    /**
     * Returns a ranked IVector of the objects in "rowGroup" 
     * that match the terms in "queryVec".
     * "queryVec" is restricted by "columnGroup".
     * Each object has to have "number" terms from "queryVec".
     * If "row" is false objects are taken from "columnGroup" and terms
     * from "rowGroup".
     */
    private IVector<IObject> basicSearch(IVector<IObject> queryVec, IMatrix matrix, IObjectGroup rowGroup, IObjectGroup columnGroup, boolean row, int number){
	if(queryVec==null || queryVec.size()==0)
	    return null;

	IObjectGroup objectGroup = rowGroup;
	IObjectGroup termGroup = columnGroup;
	if(!row){
	    objectGroup = columnGroup;
	    termGroup = rowGroup;
	}

	HashMap<IObject, HashSet<IObject>> termsForObjectMap = 
	    new HashMap<IObject, HashSet<IObject>>();
	Vector<IObject> resVec = new Vector<IObject>();
	Vector<IObject> cv = queryVec.getUnorderedVector();
	for(int i=0; i<cv.size(); i++){
	    IObject io = cv.get(i);
	    if(!termGroup.contains(io))
		continue;

	    HashSet cellSet = null;
	    if(row)
		cellSet = matrix.getRowIMatrixCellsForColumnObject(io);
	    else
		cellSet = matrix.getColumnIMatrixCellsForRowObject(io);

	    Vector cells = new Vector();
	    Iterator it = cellSet.iterator();
	    while(it.hasNext()){
		IMatrixCell mc = (IMatrixCell) it.next();
		IObject io2 = mc.getOtherIObject(io);
		if(objectGroup.contains(io2)){
		    resVec.add(io2);
		    HashSet<IObject> set = termsForObjectMap.get(io2);
		    if(set==null){
			set = new HashSet<IObject>();
			termsForObjectMap.put(io2, set);
		    }
		    set.add(io);
		}
	    }
	}

	resVec = removeObjectsWithFewTerms(resVec, termsForObjectMap, number);

	return rank(queryVec, resVec);
    }

    /**
     * Removes objects that has fewer terms than "number".
     */
    private Vector<IObject> removeObjectsWithFewTerms(Vector<IObject> resVec, HashMap<IObject, HashSet<IObject>> termsForObjectMap, int number){
	if(number>1){
	    resVec = new Vector<IObject>();
	    Iterator<Map.Entry<IObject, HashSet<IObject>>> it = 
		termsForObjectMap.entrySet().iterator();
	    while(it.hasNext()){
		Map.Entry<IObject, HashSet<IObject>> entry = it.next();
		HashSet<IObject> set = entry.getValue();
		if(set.size() >= number)
		    resVec.add(entry.getKey());
	    }
	}
	return resVec;
    }


    /**
     * Ranks the objects in "resVec" according to similarity to the terms in "queryVec".
     */
    private IVector<IObject> rank(IVector<IObject> queryVec, Vector<IObject> resVec){
	ISimilarity sim = getSimilarity(); 
	if(sim instanceof SparseISimilarity){

	    SparseISimilarity ssim = (SparseISimilarity) sim;
	    IVector<IObject> resIVec = new IVector<IObject>();
	    for(int i=0; i<resVec.size(); i++){
		IObject io = resVec.get(i);
			
		float value = ssim.sim(queryVec, ssim.getRepresentation(io));
		resIVec.add(io, value);
	    }
		    
	    return resIVec;
	}

	return null;
    }




    public IVector<IObject> search(String queryString, IMatrix matrix, IObjectGroup rowGroup, IObjectGroup columnGroup, boolean row){

	int number = (Integer) properties.getPropertyValue("Least number of terms");

	Vector<IVector<IObject>> queryVecs = new Vector<IVector<IObject>>();
	String[] qs = queryString.split(" ");
	for(int i=0; i<qs.length; i++){
	    IVector<IObject> queryVec = parseText(qs[i], matrix, rowGroup, columnGroup, row);
	    queryVecs.add(queryVec);
	}

	return multiSearch(queryVecs, matrix, rowGroup, columnGroup, row, number);
    }

    /**
     * Constructs a term vector with column objects when row is true.
     */
    public IVector<IObject> parseText(String queryString, IMatrix matrix, IObjectGroup rowGroup, IObjectGroup columnGroup, boolean row){

	IObjectGroup group = rowGroup;
	if(row)
	    group = columnGroup;

	HashMap<String, IObject> stringObjectMap = 
	    new HashMap<String, IObject>();

	Vector<IObject> ordered = group.getOrdered();
	for(int i=0; i<ordered.size(); i++){
	    IObject io = ordered.get(i);
	    stringObjectMap.put(io.getString(), io);
	}

	IVector<IObject> ret = new IVector<IObject>();
	String[] parts = queryString.split(" ");
	for(int i=0; i<parts.length; i++){
	    IObject io = stringObjectMap.get(parts[i]);
	    if(io!=null)
		ret.add(io, (float) 1);
	    else if(parts[i].startsWith("[")){
		for(int j=0; j<ordered.size(); j++){
		    io = ordered.get(j);
		    if(Pattern.matches(parts[i], io.getString()))
		       ret.add(io, (float) 1);
		}
	    }
	    //	    System.out.println(parts[i]);
	}

	return ret;
    }



}

// [d][\p{Alnum}]* by



    /*
    private IVector<IObject> basicSearch(IVector<IObject> queryVec, IMatrix matrix, IObjectGroup rowGroup, IObjectGroup columnGroup, boolean row){

	if(queryVec==null || queryVec.size()==0)
	    return null;

	IObjectGroup group = rowGroup;
	if(!row)
	    group = columnGroup;
		
		
	Vector<IObject> resVec = new Vector<IObject>();
	Vector<IObject> cv = queryVec.getUnorderedVector();
	for(int i=0; i<cv.size(); i++){
	    IObject io = cv.get(i);
	    Vector cells = matrix.getCellsWithIObjectRestrictedToGroup(io, group);
	    for(int j=0; j<cells.size(); j++){
		IMatrixCell mc = (IMatrixCell) cells.get(j);
		IObject io2 = mc.getOtherIObject(io);
		resVec.add(io2);
	    }
	}

	ISimilarity sim = similarity; //model.getISimilarity(row);
	if(sim instanceof SparseISimilarity){

	    SparseISimilarity ssim = (SparseISimilarity) sim;
	    IVector<IObject> resIVec = new IVector<IObject>();
	    for(int i=0; i<resVec.size(); i++){
		IObject io = resVec.get(i);
			
		float value = ssim.sim(queryVec, ssim.getRepresentation(io));
		resIVec.add(io, value);
	    }
		    
	    return resIVec;
	}

	return null;
    }
    */

