package infomat.io.textresult;

import mro.util.*;
import mro.util.properties.*;
import mro.util.properties.Properties;
import infomat.vectorspace.*;
import infomat.vectorspace.ordering.*;
import infomat.vectorspace.similarity.*;
import infomat.vectorspace.ivector.*;

import java.io.*;
import java.util.*; 
import java.util.zip.*;

import org.xml.sax.Attributes;
import org.xml.sax.InputSource;
import org.xml.sax.XMLReader;
import org.xml.sax.helpers.DefaultHandler;
import org.xml.sax.helpers.XMLReaderFactory;

/**
 * For exporting a confusion matrix.
 * 
 * @version          090207
 * @author           Magnus Rosell
 */
public class ConfusionMatrixExporter{

    public static void save(File file, IObjectGrouping grouping,
			    IObjectGrouping reference)
	throws IOException{

	PrintWriter pw =
	    new PrintWriter(new BufferedWriter(new BufferedWriter(new FileWriter(file))));
	save(pw, grouping, reference);
    }

    private static void save(PrintWriter pw, IObjectGrouping grouping,
			     IObjectGrouping reference)
	throws IOException{

	LatexHelper.printLatexHead(pw);

	Vector<IObjectGroup> g1groups = grouping.getGroups();
	Vector<IObjectGroup> g2groups = reference.getGroups();

	Vector<String> columnTitles = new Vector<String>();
	for(int i = 0; i < g2groups.size(); i++){
	    IObjectGroup iog2 = g2groups.get(i);
	    columnTitles.add(iog2.getString());
	}
	columnTitles.add("Total");

	boolean rowTitles = true;
	LatexHelper.printTableHead(pw, columnTitles, rowTitles);

	for(int i = 0; i < g1groups.size(); i++){
	    IObjectGroup iog1 = g1groups.get(i);
	    Vector<String> values = new Vector<String>();
	    values.add(iog1.getString());
	    for(int j = 0; j < g2groups.size(); j++){
		IObjectGroup iog2 = g2groups.get(j);
		values.add(""+iog1.numberOfSharedIObjects(iog2));
	    }
	    values.add(""+iog1.numberOfIObjects());
	    LatexHelper.printTableRow(pw, values);
	}		

	Vector<String> values = new Vector<String>();
	values.add("Total");
	for(int j = 0; j < g2groups.size(); j++){
	    IObjectGroup iog2 = g2groups.get(j);
	    values.add(""+iog2.numberOfIObjects());
	}
	values.add(""+grouping.numberOfIObjects());
	LatexHelper.printTableRow(pw, values);

	String caption = "Confusion Matrix";
	String label = caption;
	LatexHelper.printTableEnd(pw, caption, label);

	LatexHelper.printLatexEnd(pw);

	pw.flush();
    }






//     private static String xslFileName;

//     /**
//      * Saves a confusion matrix for the provided group in text format.
//      */
//     public static String saveConfusionMatrix(IObjectGrouping grouping,
// 					   IObjectGrouping reference,
// 					   File file){
// 	// xsl
// 	String resultDirectoryName = 
// 	    result.getCanonicalFile().getParentFile().getPath();
	
// 	xslFileName = resultDirectoryName+separator+"confusionMatrix.xml";
// 	File xslFile = new File(xslFileName);
// 	PrintWriter outxsl =
// 	    new PrintWriter(new BufferedWriter(new BufferedWriter(new FileWriter(xslFile))));
// 	printXSL(outxsl);

// 	PrintWriter out =
// 	    new PrintWriter(new BufferedWriter(new BufferedWriter(new FileWriter(file))));
// 	save(outxsl);

//     } 

//     private static DoubleHashMap constructConfusionMatrix(IObjectGrouping evalGrouping, IObjectGrouping refGrouping){
// 	DoubleHashMap confusionMatrix = new DoubleHashMap();
// 	Vector<IObjectGroup> g1groups = evalGrouping.getGroups();
// 	Vector<IObjectGroup> g2groups = refGrouping.getGroups();

// 	for(int i = 0; i < g1groups.size(); i++){
// 	    IObjectGroup iog1 = g1groups.get(i);
// 	    for(int j = 0; j < g2groups.size(); j++){
// 		IObjectGroup iog2 = g2groups.get(j);
// 		confusionMatrix.put(iog1, iog2, iog1.numberOfSharedIObjects(iog2));
// 	    }
// 	}		
// 	return confusionMatrix;
//     }



//     private static void save(PrintWriter pw, IObjectGrouping grouping,
// 			     IObjectGrouping reference)
// 	throws IOException{

// 	// XML
// 	XMLHelper.printXMLHead(out);
// 	pw.println("<?xml-stylesheet type=\"text/xsl\" href=\""+xslFileName+"\"?>");
// 	printGroup(pw);

// 	/*
// 	DoubleHashMap confusionMatrix = 
// 	    constructConfusionMatrix(grouping, reference);
// 	*/

// 	Vector<IObjectGroup> g1groups = grouping.getGroups();
// 	Vector<IObjectGroup> g2groups = reference.getGroups();

// 	for(int i = 0; i < g1groups.size(); i++){
// 	    IObjectGroup iog1 = g1groups.get(i);
// 	    pw.print();

// 	    for(int j = 0; j < g2groups.size(); j++){
// 		IObjectGroup iog2 = g2groups.get(j);
// 		confusionMatrix.put(iog1, iog2, iog1.numberOfSharedIObjects(iog2));
// 	    }
// 	}		




// 	out.flush();
//     }



//     /**
//      * Prints a basic XSL-file for IObjectGroup XML-files to a 
//      * {@link java.io.PrintWriter}.
//      */
//     public void printXSL(PrintWriter pw) throws IOException{
// 	pw.println("<?xml version=\"1.0\" encoding=\"ISO-8859-1\"?>");
// 	pw.println("<xsl:stylesheet version=\"1.0\"");
// 	pw.println("xmlns:xsl=\"http://www.w3.org/1999/XSL/Transform\">");
// 	pw.println("<xsl:template match=\"/\">");
// 	pw.println("<html>");
// 	pw.println("<body>");
// 	pw.println("<xsl:apply-templates/>");
// 	pw.println("</body>");
// 	pw.println("</html>");
// 	pw.println("</xsl:template>");
// 	pw.println("");

// 	pw.println("<xsl:template match=\"InfomatIObjectGroupTextGeneralInfo\">");
// 	pw.println("<h2><xsl:value-of select=\"IObjectGroupName\"/></h2>");
// 	pw.println("<xsl:value-of select=\"NumberOfIObjects\"/> IObjects.");
// 	pw.println("<xsl:value-of select=\"NumberOfRepresentationIObjects\"/> Representation IObjects.");
// 	pw.println("<br />");
// 	pw.println("</xsl:template>");
// 	pw.println("");
	
// 	pw.println("</xsl:stylesheet>");
// 	pw.println("");
//     }




//     public static void printDTD(PrintWriter out) {
// 	out.println("<!DOCTYPE InfomatConfusionMatrix [");

// 	out.println("<!ELEMENT InfomatConfusionMatrix (IObjectGroupColumTitles, IObjectGroupRow*)>");
// 	out.println("<!ELEMENT IObjectGroupColumnTitles (IObjectGroupColumnTitle*)>");
// 	out.println("<!ELEMENT IObjectGroupColumnTitle (#PCDATA)>");
// 	out.println("<!ELEMENT IObjectGroupRow (IObjectGroupRowTitle, InfomatConfusionMatrixElement)>");
// 	out.println("<!ELEMENT IObjectGroupRowTitle (#PCDATA)>");
// 	out.println("<!ELEMENT InfomatConfusionMatrixElement (#PCDATA)>");
// 	out.println("]>");
//     }




//     public <T extends IVectorIndex> void savePartGroupFile(PrintWriter out, IObjectGroup group, String groupLocation, IVector<T> descriptionVec, ISimilarity similarity, String ioUnknownLocation) throws IOException{

// 	int exportNumberOfDescriptionObjects = (Integer) properties.getPropertyValue("Number of Description IObjects For Group");

// 	int exportNumberOfRepresentativeObjects = (Integer) properties.getPropertyValue("Number of Representation IObjects For Group");

// 	int exportNumberOfObjects = (Integer) properties.getPropertyValue("Number of IObjects For Group");

// 	out.println("<InfomatIObjectGroupText>");
// 	out.println("<InfomatIObjectGroupTextGeneralInfo>");
// 	out.println("<IObjectGroupName>"+group.getName()+"</IObjectGroupName>");
// 	if(groupLocation!=null){
// 	    out.println("<IObjectGroupLocation>"+groupLocation+"</IObjectGroupLocation>");
// 	}

// 	out.println("<NumberOfIObjects>"+group.getSortedObjectOrderValueVector().size()+"</NumberOfIObjects>");
	
// 	if(descriptionVec!=null){
// 	    out.println("<NumberOfDescriptionIObjects>"+descriptionVec.size()+"</NumberOfDescriptionIObjects>");
// 	}

// 	SparseISimilarity sis = null;
// 	if(similarity instanceof SparseISimilarity){
// 	    sis = (SparseISimilarity) similarity;
// 	}
// 	if(sis!=null){
// 	    out.println("<NumberOfRepresentationIObjects>"+sis.getGroupRepresentation(group).size()+"</NumberOfRepresentationIObjects>");
// 	}
// 	out.println("</InfomatIObjectGroupTextGeneralInfo>");

// 	if(descriptionVec!=null){
// 	    out.println("<DescriptionIObjects>");
// 	    IVectorTextResult.saveIVector(out, descriptionVec, exportNumberOfDescriptionObjects, ioUnknownLocation);
// 	    out.println("</DescriptionIObjects>");
// 	}

// 	if(sis!=null){
// 	    IVector<IVectorIndex> reprVec = sis.getGroupRepresentation(group);

// 	    out.println("<RepresentationIObjects>");
// 	    IVectorTextResult.saveIVector(out, reprVec, exportNumberOfRepresentativeObjects, ioUnknownLocation);
// 	    out.println("</RepresentationIObjects>");
// 	}

// 	IVector<IObject> objVec = group.getIVector();
// 	out.println("<IObjects>");
// 	IVectorTextResult.saveIVector(out, objVec, exportNumberOfObjects, ioUnknownLocation);
// 	out.println("</IObjects>");

// 	out.println("</InfomatIObjectGroupText>");

// 	out.flush();
//     }








//     public static void printPartOfGroupingXSL(PrintWriter pw, String context) throws IOException{
// 	pw.println("<xsl:template match=\""+context+"InfomatIObjectGroupText\">");
// 	pw.println("<li>");
// 	pw.println("<xsl:apply-templates select=\"InfomatIObjectGroupTextGeneralInfo\"/> ");
// 	pw.println("<br />");
// 	pw.println("<xsl:apply-templates select=\"DescriptionIObjects/IVector\"/> ");
// 	pw.println("<xsl:apply-templates select=\"RepresentationIObjects/IVector\"/> ");
// 	pw.println("<xsl:apply-templates select=\"IObjects/IVector\"/> ");

// 	pw.println("</li>");
// 	pw.println("</xsl:template>");
// 	pw.println("");



// 	pw.println("<xsl:template match=\"InfomatIObjectGroupTextGeneralInfo\">");

// 	pw.println("<xsl:element name=\"a\">");
// 	pw.println("<xsl:attribute name=\"href\">");
// 	pw.print("file:///");
// 	pw.println("<xsl:value-of select=\"IObjectGroupLocation\"/>");
// 	pw.println("</xsl:attribute>");
// 	pw.println("<xsl:value-of select=\"IObjectGroupName\"/>");
// 	pw.println("</xsl:element>");
// 	pw.println("&#160;");
// 	pw.println("<xsl:value-of select=\"NumberOfIObjects\"/> IObjects.");
// 	pw.println("<xsl:value-of select=\"NumberOfRepresentationIObjects\"/> Representation IObjects.");

// 	pw.println("</xsl:template>");
// 	pw.println("");






// 	/*
// 	pw.println("<xsl:template match=\""+context+"InfomatIObjectGroupTextGeneralInfo\">");
// 	//	pw.println("<li>");

// 	pw.println("<xsl:element name=\"a\">");
// 	pw.println("<xsl:attribute name=\"href\">");
// 	pw.println("<xsl:value-of select=\"IObjectGroupLocation\"/>");
// 	pw.println("</xsl:attribute>");
// 	pw.println("<xsl:value-of select=\"IObjectGroupName\"/>");
// 	pw.println("</xsl:element>");
// 	pw.println("&#160;");
// 	pw.println("<xsl:value-of select=\"NumberOfIObjects\"/> IObjects.");
// 	pw.println("<xsl:value-of select=\"NumberOfRepresentationIObjects\"/> Representation IObjects.");
// 	pw.println("<br />");

// 	//	pw.println("</li>");

// 	pw.println("</xsl:template>");
// 	pw.println("");
// 	*/
	
//     }





}
