package infomat.io.textword;

import infomat.vectorspace.*;
import infomat.vectorspace.similarity.*;
import infomat.vectorspace.ivector.*;

import java.io.*;
import java.util.*; 
import java.util.zip.*;

import org.xml.sax.Attributes;
import org.xml.sax.InputSource;
import org.xml.sax.XMLReader;
import org.xml.sax.helpers.DefaultHandler;
import org.xml.sax.helpers.XMLReaderFactory;

/**
 * For constructing a text-word-IMatrix. 
 * Used in the import classes.
 * 
 * @version          080605
 * @author           Magnus Rosell
 */
public class TextWordIMatrixConstructor{

    // The InfomatModel that is loaded/saved
    //    private InfomatModel im;
    private IMatrix matrix;
    private HashMap<String, IVector<IVectorIndex>> stringProj;
    private HashMap<String, IObject> stringWordObjectMap;

    public TextWordIMatrixConstructor(){
	initialize(null, null);
    }

    public TextWordIMatrixConstructor(IMatrix wordProjectionMatrix,
				      SparseISimilarity sim){
	initialize(wordProjectionMatrix, sim);
    }

    public IMatrix getIMatrix(){
	return matrix;
    }

    private void initialize(IMatrix wordProjectionMatrix, SparseISimilarity sim){
	if(wordProjectionMatrix==null){
	    matrix = new IMatrix("Words", "Texts");
	    stringWordObjectMap = new HashMap<String, IObject>();
	}else{
	    matrix = new IMatrix("Word Groups", "Texts");
	    IObjectSet colSet = wordProjectionMatrix.getColumnIObjectSet();
	    Vector<IObject> projGroups = colSet.getOrdered();
	    for(int i=0; i<projGroups.size(); i++){
		IObjectGroup group = (IObjectGroup) projGroups.elementAt(i);
		matrix.addColumnIObject(group);

		stringProj = new HashMap<String, IVector<IVectorIndex>>();
		Vector<IObject> ios = group.getOrdered();
		for(int j=0; j<ios.size(); j++){
		    IObject io = ios.elementAt(j);
		    IVector<IVectorIndex> vec = 
			sim.getRepresentation(io);
		    stringProj.put(io.getString(), vec);
		}
	    }
	}
    }

    IObject textObject;
    IObject wordObject = null;
    int id = 0;

    public void addText(String s, String location){
	textObject = new IObject(s, location, id);
	matrix.addRowIObject(textObject);
	id++;
    }

    public void addWord(String wordString, String location,  
			int localCount, float localWeight, float weight){

	if(stringProj==null){
	    
	    IObject wordObject = stringWordObjectMap.get(wordString);
	    if(wordObject==null){

		if(location!=null)
		    wordObject = new IObject(wordString, location, id);
		else
		    wordObject = new IObject(wordString, id);
		id++;
		stringWordObjectMap.put(wordString, wordObject);
		matrix.addColumnIObject(wordObject);
	    }
	    IMatrixCell mc = matrix.getCell(textObject, wordObject);
	    if(mc==null){
		mc = new IMatrixCell(textObject, wordObject, 
				     localCount, localWeight, weight);
		matrix.addCell(mc);
	    }else{
		mc.addToLocalCount(localCount);
	    }
	    
	}else{

	    
	    IVector<IVectorIndex> vec = stringProj.get(wordString);
	    
	    Iterator<Map.Entry<IVectorIndex, Float>> it = 
		vec.getMap().entrySet().iterator();
	    while(it.hasNext()){
		Map.Entry<IVectorIndex, Float> entry = it.next();
		IObject reprIO = (IObject) entry.getKey();
		float factor = entry.getValue();
		
		IMatrixCell mc = matrix.getCell(textObject, reprIO);
		if(mc==null){
		    mc = new IMatrixCell(textObject, reprIO, 
					 localCount*factor, 1);
		    matrix.addCell(mc);
		}else{
		    mc.addToLocalCount(localCount*factor);
		}
	    }
    	}
    }



}
