package infomat.vectorspace;

import infomat.io.*;
import infomat.vectorspace.ivector.*;

import mro.util.*;
import mro.util.properties.*;

import java.lang.Math.*;
import java.io.*;
//import java.util.*;
import java.util.HashSet;
import java.util.HashMap;
import java.util.Map;
import java.util.Vector;
import java.util.Iterator;


/**
 * A stoplist. Consists of an IVector of String:s. 
 * To stop (remove) IObject:s these String:s are compared to 
 * a IMatrix along a specified dimension (rows or columns).
 *
 * @version          080721
 * @author           Magnus Rosell
 */
public class Stoplist implements HasProperties{

    Properties properties;


    public mro.util.properties.Properties getDefaultProperties(){

	mro.util.properties.Properties properties = 
	    new mro.util.properties.Properties("Stoplist Properties", CurrentClassGetter.getClassName());
		
	try{
			
	    Property p = new Property("Remove words shorter than", 3, "Remove words shorter than");
	    properties.addProperty(p);

	    p = new Property("Remove words longer than", 20, "Remove words longer than");
	    properties.addProperty(p);


	    p = new Property("Remove numbers", true, "Remove numbers");
	    PropertyValueDescription pvd = new PropertyValueDescription(true, "Do it.");
	    p.addValueDescription(pvd);
	    pvd = new PropertyValueDescription(false, "Do not do it.");
	    p.addValueDescription(pvd);
	    properties.addProperty(p);

	    p = new Property("Remove white spaces", true, "Remove white spaces");
	    pvd = new PropertyValueDescription(true, "Do it.");
	    p.addValueDescription(pvd);
	    pvd = new PropertyValueDescription(false, "Do not do it.");
	    p.addValueDescription(pvd);
	    properties.addProperty(p);

			
	}catch(PropertyNameConflictException pnce){
	    // Won't happen here
	}
	return properties;
    }


    private IVectorStringIO stopStringIO = new IVectorStringIO();
    private IVector<IVectorString> stopStrings = new IVector<IVectorString>();
    private IVector<IObject> stopIObjects = new IVector<IObject>();


    public Stoplist(){
	properties = getDefaultProperties();
    }

    public Stoplist(Properties properties){
	this.properties = properties;
    }

    public Stoplist(String fileName){
	readFile(fileName);
	properties = getDefaultProperties();
    }

    public Stoplist(Properties properties, String fileName){
	readFile(fileName);
	this.properties = properties;
    }

    public IVector<IVectorString> getStopStringsVector(){
	return stopStrings;
    }

    public IVector<IObject> getStopIObjectsVector(){
	return stopIObjects;
    }

    /**
     * Very unsafe!
     */
    public void setStopIObjectsVector(IVector<IObject> vec){
	stopIObjects=vec;
    }

    /**
     * Adapts the stoplist to the specified matrix and dimension.
     * Only IOjbect:s corresponding to any string are stored 
     * (and later removed).
     */
    public void constructStopIObjectsFromStopStrings(IMatrix matrix, 
						     boolean row){
	boolean removeNumbers = (Boolean) getPropertyValue("Remove numbers");
	boolean removeWhiteSpaces = (Boolean) getPropertyValue("Remove white spaces");
	int shorterThan = (Integer) getPropertyValue("Remove words shorter than");
	int longerThan = (Integer) getPropertyValue("Remove words longer than");

	IObjectSet ioset = matrix.getColumnIObjectSet();
	if(row)
	    ioset = matrix.getRowIObjectSet();

	HashSet<String> tempStoplist = new HashSet<String>();
	IVector<IVectorString> ivec = getStopStringsVector();
	Vector<IVectorString> vec = ivec.getOrderedVector();
	for(int i=0; i<vec.size(); i++){
	    IVectorString ii = vec.get(i);
	    tempStoplist.add(ii.getString());
	}

	stopIObjects = new IVector<IObject>();
	//	Vector<IObject> removed = new Vector<IObject>();
	Vector<IObject> ios = ioset.getOrdered();

	//	float numberOfIObjects = ioset.numberOfIObjects();
	float tmp=0.0f;
	for(int i=0; i<ios.size(); i++){
	    IObject io = ios.elementAt(i);
	    String s = io.getAmbiguousString();

	    if(tempStoplist.contains(s)){ // stoplist
		//		stopIObjects.add(io, numberOfIObjects/(i+1));
		stopIObjects.put(io, tmp);
	    }else if(s.length()<shorterThan){
		//		stopIObjects.add(io, numberOfIObjects/(i+1));
		stopIObjects.put(io, tmp);
	    }else if(s.length()>longerThan){
		//		stopIObjects.add(io, numberOfIObjects/(i+1));
		stopIObjects.put(io, tmp);
	    }
	    if(removeWhiteSpaces){ // white spaces
		if(s.equals(""))
		    //		    stopIObjects.add(io, numberOfIObjects/(i+1));
		    stopIObjects.put(io, tmp);
	    }
	    if(removeNumbers){ // numbers
		try{
		    Double.parseDouble(s);
		    //		    stopIObjects.add(io, numberOfIObjects/(i+1));
		    stopIObjects.put(io, tmp);
		}catch(NumberFormatException e){
		    // do nothing
		}
	    }
	}

	stopIObjects.sortLiteral();
	
	/*
	float numberOfRemoved = removed.numberOfIObjects();
	for(int i=0; i<removed.size(); i++){
	    IObject io = removed.elementAt(i);
	    stopIObjects.add(io, numberOfRemoved-i);
	}
	*/

	/*
	System.out.println("stoplist: "+stoplist.size());
	System.out.println("stopIObjects: "+stopIObjects.size());
	*/
    }

    /**
     * Constructs a stoplist of String:s from the current set of IObjects.
     */
    public void constructStopStringsFromStopIObjects(){
	/*
	//	stoplist = new HashSet<String>();
	stopStrings = new IVector<IVectorString>();
	Vector<IObject> vec = stopIObjects.getUnorderedVector();
	for(int i=0; i<vec.size(); i++){
	    //	    IVectorIndex ii = vec.get(i);
	    IObject io = vec.get(i);
	    String s = io.getAmbiguousString();
	    stopStrings.add(new IVectorString(s), stopIObjects.getValue(io));
	    //	    stoplist.add(s);
	}
	*/

	HashMap<String, IObject> tmpStopMap = new HashMap<String, IObject>();
	Vector<IObject> vec = stopIObjects.getUnorderedVector();
	for(int i=0; i<vec.size(); i++){
	    IObject io = vec.get(i);
	    String s = io.getAmbiguousString();
	    tmpStopMap.put(s, io);
	}
	
	stopStrings = new IVector<IVectorString>();
	Iterator<Map.Entry<String, IObject>> it = 
	    tmpStopMap.entrySet().iterator();
	while(it.hasNext()){
	    Map.Entry<String, IObject> entry = it.next();
	    stopStrings.add(new IVectorString(entry.getKey()), stopIObjects.getValue(entry.getValue()));
	}

    }


    /**
     * Removes the IObject:s in the stoplist from the matrix along the
     * specified dimension. (If the matrix has the same set as 
     * rows and columns
     * the IObjects are removed from both.)
     */
    public void removeStopIObjects(IMatrix matrix, boolean row){

	System.out.println("Removing Stop-IObjects...");

	IObjectSet ioset = matrix.getColumnIObjectSet();
	if(row)
	    ioset = matrix.getRowIObjectSet();
	
	Vector<IObject> vec = stopIObjects.getUnorderedVector();
    
	for(int i=0; i<vec.size(); i++){
	    /*
	    IVectorIndex ii = vec.elementAt(i);
	    IObject rIO = (IObject) ii;
	    ioset.remove(rIO);
	    */
	    IObject rIO = vec.get(i);
	    ioset.remove(rIO);

	    HashSet set = matrix.getRowIMatrixCellsForColumnObject(rIO);
	    if(row){
		set = matrix.getColumnIMatrixCellsForRowObject(rIO);
	    }
	    Iterator it = set.iterator();
	    while(it.hasNext()){
		IMatrixCell mc = (IMatrixCell) it.next();
		matrix.removeCell(mc);
	    }
	}
	//	ioset.purgeGroupingsToSet();
	//	matrix.purgeMatrixToCurrentGroupings();
	matrix.purgeMatrixToGroups(matrix.getRowIObjectSet(), matrix.getColumnIObjectSet());
    }


    /**
     * Applies #constructStopIObjectsFromStopStrings 
     * and #removeStopIObjects().
     */
    public void removeIObjectsInStoplist(IMatrix matrix, boolean row){
	constructStopIObjectsFromStopStrings(matrix, row);
	removeStopIObjects(matrix, row);


	/*
	for(int i=0; i<removed.size(); i++){
	    IObject rIO = removed.elementAt(i);
	    ioset.remove(rIO);

	    HashSet set = matrix.getRowIMatrixCellsForColumnObject(rIO);
	    if(row){
		set = matrix.getColumnIMatrixCellsForRowObject(rIO);
	    }
	    Iterator it = set.iterator();
	    while(it.hasNext()){
		IMatrixCell mc = (IMatrixCell) it.next();
		matrix.removeCell(mc);
	    }
	}
	ioset.purgeGroupingsToSet();
	*/
	/*
	  Iterator<String> it = stoplist.iterator();
	  while(it.hasNext()){
	  ioset.remove(it.next());
	  }
	*/
    }

    /**
     * Reads strings from a text file with one string per row.
     */
    public void readFile(String fileName){
	stopStrings = stopStringIO.readFile(fileName);
	//	readFile(new File(fileName));
    }

    public void readFile(File file){
	stopStrings = stopStringIO.readFile(file);

	/*
	stopStrings = new IVector<IVectorString>();
	try{
	    BufferedReader bf = new BufferedReader(new FileReader(file));
	    String s = bf.readLine();
	    int i=0;
	    while(s!=null){
		//		stoplist.add(s);
		stopStrings.add(new IVectorString(s), (float) 100.0/(i+1));
		i++;
		s = bf.readLine();
	    }
	    bf.close();
	}catch(IOException ioe){
	    System.out.println("Stoplist: Bad IO!");
	}
	*/
    }


    /**
     * Write strings to a text file with one string per row.
     */
    public void writeFile(String fileName){
	stopStringIO.writeFile(stopStrings, fileName);
	//	writeFile(new File(fileName));
    }

    public void writeFile(File file){
	stopStringIO.writeFile(stopStrings, file);
	/*
	try{
	    PrintWriter out =
		new PrintWriter(new BufferedWriter(new BufferedWriter(new FileWriter(file))));
	    
	    Vector<IVectorString> vec = stopStrings.getOrderedVector();
	    for(int i=0; i<vec.size(); i++){
		IVectorIndex ii = vec.get(i);
		out.println(ii.getString());
	    }

	    out.close();
	}catch(IOException ioe){
	    System.out.println("Stoplist: Bad IO!");
	}
	*/
    }



    public Vector getPropertyVector(){
    	return properties.getPropertyVector();
    }
    

    public Object getPropertyValue(String name){
    	return properties.getPropertyValue(name);
    }
    public void setProperty(String name, Object value) throws PropertyNotFoundException{
    	properties.setProperty(name, value);
    }

    public mro.util.properties.Properties getProperties(){
    	return properties;
    }

    public void setProperties(mro.util.properties.Properties properties){
    	this.properties=properties;
    }

}
