package infomat.vectorspace.ri;

import infomat.vectorspace.*;

import java.io.*;
import java.util.HashMap;
import java.util.LinkedList;


/**
 *
 * @version          091005
 * @author           Magnus Rosell
 */
public class RIIO{
    
    private RI ri;

    /*

    public RIIO(RI ri){
	this.ri=ri;
    }
    */

    public RI load(File path){
	RI ri = new RI();
	recurseFile(path, ri);
	return ri;
    }
    

    private void recurseFile(File file, RI ri) {
	AllFileFilter aff = new AllFileFilter();

	if(file.isDirectory()){
	    File[] list = file.listFiles(aff);
	    for(int i=0; i<list.length; i++){
		recurseFile(list[i], ri);
	    }
	}else{
	    doOneFile(file, ri);
	}
    }

    private HashMap<String, IObject> stringWordObjectMap = 
	new HashMap<String, IObject>();

    private void doOneFile(File file, RI ri){

	try {

	    RIWordWindowWeighting wordWindowWeighting = 
		new ConstantWordWindowWeighting();

	    int leftWindowSize = 2;
	    int rightWindowSize = 2;
	    int largestWindowSize = leftWindowSize;
	    if(rightWindowSize>largestWindowSize)
		largestWindowSize = rightWindowSize;

	    String[] leftWindow = new String[leftWindowSize];
	    String focusWord = null;
	    String[] rightWindow = new String[rightWindowSize];

	    LinkedList<String> contextList = new LinkedList<String>();

	    BufferedReader in = 
		new BufferedReader(new FileReader(file));
	    String s=in.readLine();
	    while(s!=null){
		String[] words = s.split(" ");
		for(int i=0; i<words.length; i++){
		    
		    //		    System.out.println(words[i]);

		    String newWord = words[i]; 
		    contextList.add(newWord);
		    ri.addNewWord(newWord);

		    if(contextList.size()>1){
			if(contextList.size()>largestWindowSize)
			    contextList.removeFirst();
			int distance = contextList.size();
			for(int j=0; j<contextList.size(); j++){
			    String word = contextList.get(j);

			    int pos = contextList.size()-j;
			    if(pos<=leftWindowSize)
				ri.add(newWord, word, wordWindowWeighting.weight(-pos));
			    if(pos<=rightWindowSize)
				ri.add(word, newWord, wordWindowWeighting.weight(pos));
			       

			}


		    }



		}

		s=in.readLine();
	    }
	    
	    in.close();
	    

	}catch(IOException ioE) {
	    System.out.println(ioE);
	    ioE.printStackTrace();
	}
    }



private class TextFileFilter implements FileFilter{

    public boolean accept(File pathname){
	if(pathname.isDirectory())
	    return true;
	if(pathname.getName().endsWith(".txt") ||
	   pathname.getName().endsWith(".TXT"))
	    return true;
	return false;
    }

}

private class AllFileFilter implements FileFilter{

    public boolean accept(File pathname){
	return true;
    }

}


}

