package infomat.vectorspace.similarity;

import infomat.algorithms.grouping.NeedsIMatrix;
import infomat.algorithms.grouping.NeedsIMatrixDimension;
import infomat.vectorspace.*;
import infomat.vectorspace.ivector.*;

import mro.util.*;
import mro.util.properties.*;

import java.util.*;

/**
 * The literal similarity.
 *
 * @version          080811
 * @author           Magnus Rosell
 */
public class SparseLiteralSimilarity extends SparseISimilarity{

    public mro.util.properties.Properties getDefaultProperties(){

	mro.util.properties.Properties properties = 
	    new mro.util.properties.Properties("Literal Similarity Properties", CurrentClassGetter.getClassName());
		
	    /*
	try{
	    Property p = new Property("Normalize", false, "Wether normalization is used. This can also be done through the Weighter.");
	    PropertyValueDescription pvd = 
		new PropertyValueDescription(true, "Do it.");
	    p.addValueDescription(pvd);
	    pvd = new PropertyValueDescription(false, "Do not do it.");
	    p.addValueDescription(pvd);
	    properties.addProperty(p);
	}catch(PropertyNameConflictException pnce){
	    // Won't happen here
	}
	    */
	return properties;
    }

    public String getName(){
    	return "Literal Similarity";
    }

    public Vector getPropertyVector(){
    	return properties.getPropertyVector();
    }
    

    public Object getPropertyValue(String name){
    	return properties.getPropertyValue(name);
    }

    public void setProperty(String name, Object value) throws PropertyNotFoundException{
    	properties.setProperty(name, value);
	/*
	if(name.compareTo("Normalize")==0)
	    normalize = (Boolean) getPropertyValue("Normalize");
	*/
    }

    public mro.util.properties.Properties getProperties(){
    	return properties;
    }
    public void setProperties(mro.util.properties.Properties properties){
    	this.properties=properties;
    }


    //    private Boolean normalize = false;

    public SparseLiteralSimilarity(){
	super();
	//	this.normalize = (Boolean) getPropertyValue("Normalize");
    }

    public SparseLiteralSimilarity(SparseLiteralSimilarity sim){
	super(sim);
    }

    /**
     * Calculates the similarity between two IObject:s.
     * Extracts the representation from the matrix.
     */
    public float sim(IObject io1, IObject io2){

	IVector<IVectorIndex> vec1 = getRepresentation(io1);
	IVector<IVectorIndex> vec2 = getRepresentation(io2);

	return sim(vec1, vec2);
    }

    /**
     * Calculates the similarity between IObject and a IObjectGroup.
     * Extracts the representation from the matrix.
     * Uses the group representation for IObjectGroup:s of all kinds.
     * This takes more time to extract. Therefore it might be better
     * to use the #getGroupRepresentation-method instead...
     */
    public float sim(IObject io, IObjectGroup iog){

	IVector<IVectorIndex> ioVec = getRepresentation(io);
	IVector<IVectorIndex> groupVec = getRepresentation(iog);

	return sim(ioVec, groupVec);
    }

    /**
     * Calculates the similarity between two IObjectGroup:s.
     * Extracts the representation from the matrix.
     * Uses the group representation for IObjectGroup:s of all kinds.
     * This takes more time to extract. Therefore it might be better
     * to use the #getGroupRepresentation-method instead...
     */
    public float sim(IObjectGroup iog1, IObjectGroup iog2){

	IVector<IVectorIndex> vec1 = getGroupRepresentation(iog1);
	IVector<IVectorIndex> vec2 = getGroupRepresentation(iog2);

	return sim(vec1, vec2);
    }


	
    /**
     * Calculates the similarity between two representations.
     *
     * <br><br>
     * Checks whether the second representation is smaller
     * than first. A small decrease in time is achieved by 
     * giving the representation that one believes is smaller 
     * as first parameter.
     * (Do not check it also. That increaes time spent.)
     */
    public <U extends IVectorIndex, V extends IVectorIndex> float sim(IVector<U> vec1, IVector<V> vec2){

	String s1 = ((U) vec1.getUnorderedVector().get(0)).getString();
	String s2 = ((V) vec2.getUnorderedVector().get(0)).getString();
	int length = s1.length();
	int length1 = s1.length();
	int length2 = s2.length();
	if(length2 < length)
	    length=length2;

	int match=0;
	while(match<length && s1.charAt(match) == s2.charAt(match))
	    match++;
	return ((float) 2*match)/(length1+length2);
    }



    /**
     * Returns the representation in the form of a IVector<IObject>.
     */
    public IVector<IVectorIndex> getRepresentation(IObject io){

	IVector<IVectorIndex> representation = new IVector<IVectorIndex>();
	representation.put(io, 1.0f);
	return representation;
    }



	

    public IVector<IVectorIndex> getGroupRepresentation(IObjectGroup iog){
	IVector<IVectorIndex> centroid = new IVector<IVectorIndex>();
	if(iog.numberOfIObjects()==1)
	    centroid.put(iog.getOrdered().get(0), 1.0f);
	else
	    centroid.put(iog, 1.0f);
	return centroid;
    }


}



