package infomat.view.algorithms;

import infomat.algorithms.grouping.*;
import infomat.io.*;
import infomat.vectorspace.*;
import infomat.vectorspace.similarity.*;
import infomat.view.*;
import infomat.view.selector.*;
import infomat.view.util.*;
import infomat.view.grouping.settings.*;
import mro.util.properties.*;
import mro.util.properties.Properties;
import mro.util.properties.dialog.*;

import java.awt.*;
import java.awt.Dialog.*;//ModalityType;
import java.awt.event.*;
import javax.swing.border.*;
import java.util.*;
import java.io.*;

import javax.swing.*;


	
/**
 * A JFrame for clustering. Presents the properties of a Clustering Algorithm
 * and lets the user cluster the rows or columns accoring to it.
 *
 * @see AlgorithmSelectionPanel
 *
 * @version          080409
 * @author           Magnus Rosell
 */
public class AlgorithmFrame extends JFrame
    implements ActionListener{

    public Dimension ALGORITHMFRAME_PREFERRED_SIZE = 
	new Dimension(350, 600);

    private JPanel algorithmPanel;
    private AlgorithmSelectionPanel algorithmSelectionPanel;
	
    private PropertiesPanel algPropertiesPanel;
    private GroupingAlgorithm algorithm;
    //    private IMatrix matrix;
    private InfomatModel model;
    private InfomatView view;
		
		
    private AlternatingButton rowsOrColumns;
    //    private JButton rowsOrColumns;
    private JButton apply;
    private JButton cancel;
		


    public AlgorithmFrame(InfomatModel model, InfomatView view){
	super("Clustering Algorithm");//algorithm.getAlgorithmName());

	this.model=model;
	this.view=view;
			
	if(model.getIMatrix()==null || view==null || model==null){
	    dispose();
	    return;
	}


	algorithmSelectionPanel = new AlgorithmSelectionPanel(this);
	algorithm = algorithmSelectionPanel.getSelectedAlgorithm();

	String[] alternatives = {"Rows", "Columns"};
	//	AlternatingButton alt = new AlternatingButton(alternatives);
	//	rowsOrColumns = alt;
	rowsOrColumns = new AlternatingButton(alternatives);
	rowsOrColumns.addActionListener(this);

	setVisible(true);
	setPreferredSize(ALGORITHMFRAME_PREFERRED_SIZE);

	algorithmPanel = new JPanel();
	algorithmPanel.setPreferredSize(ALGORITHMFRAME_PREFERRED_SIZE);
	algorithmPanel.setLayout(new BoxLayout(algorithmPanel, BoxLayout.Y_AXIS));

	putAll();
	add(algorithmPanel);
	pack();
    }

    public void putAll(){
	//	removeAll();
	algorithmPanel.removeAll();
	
	Container container = getContentPane();

			
			

	JTextField dummy = new JTextField("Apply");
	JButton dummyButton = new JButton("Cancel");
	int height = (int) dummy.getPreferredSize().getHeight();
	int width = (int) dummyButton.getPreferredSize().getWidth();
	Dimension buttonSize = new Dimension(width, height);
			
	JPanel infoAndButtons = new JPanel();
	infoAndButtons.setLayout(new GridLayout(3,1));
	JLabel info = new JLabel("Alter properties below and press \"Apply\"");
	infoAndButtons.add(info);
			
	JPanel dimPanel = new JPanel();
	dimPanel.setLayout(new BoxLayout(dimPanel, BoxLayout.X_AXIS));
	JLabel dimInfo = new JLabel("Choose rows or columns: ");
	dimPanel.add(dimInfo);

	rowsOrColumns.setPreferredSize(buttonSize);
	dimPanel.add(rowsOrColumns);
	infoAndButtons.add(dimPanel);
			
	JPanel buttonPanel = new JPanel();
	buttonPanel.setLayout(new BoxLayout(buttonPanel, BoxLayout.X_AXIS));
	apply = new JButton("Apply");
	apply.setPreferredSize(buttonSize);
	apply.addActionListener(this);
	buttonPanel.add(apply);
	cancel = new JButton("Cancel");
	cancel.setPreferredSize(buttonSize);
	cancel.addActionListener(this);
	buttonPanel.add(cancel);
	infoAndButtons.add(buttonPanel);
	infoAndButtons.setBorder(new TitledBorder("Clustering Algorithm")); //algorithm.getAlgorithmName()));
	algorithmPanel.add(infoAndButtons);
			
			
	JPanel algChoicePanel = new JPanel();
	algChoicePanel.add(algorithmSelectionPanel);
	//	JLabel description = new JLabel("Description: "+algorithm.getInfoString());
	//	algChicePanel.add(description);

	algChoicePanel.setBorder(new TitledBorder("Choose algorithm"));
	algorithmPanel.add(algChoicePanel);
					
	String path = (String) model.getPropertyValue("Result Path");
	algPropertiesPanel = new PropertiesPanel(algorithm, path);
	algorithmPanel.add(algPropertiesPanel);
			
	//setSize(ALGORITHMFRAME_PREFERRED_SIZE);
	//setMinimumSize(ALGORITHMFRAME_PREFERRED_SIZE);
	//	pack();

	//	add(algorithmPanel);
	//	pack();
	algorithmPanel.revalidate();
	algorithmPanel.repaint();
	repaint();
    }
		
    public void actionPerformed(ActionEvent e) {
	if(e.getSource() instanceof JButton){
	    JButton source = (JButton) e.getSource();
	    if(source == rowsOrColumns){
		rowsOrColumns.press();
		algorithmPanel.repaint();
		repaint();
	    }else if(source == apply){
		ProgressFrame pf = new ProgressFrame("Clustering - "+algorithm.getAlgorithmName());
					
		boolean row=true;
		if(rowsOrColumns.getText().equals("Columns"))
		    row=false;

		/*
		IObjectGroup group = model.getIMatrix().getColumnIObjectSet();
		if(row)
		    group = model.getIMatrix().getRowIObjectSet();
		*/
		
		IObjectGroup group = model.getIMatrix().getColumnIObjectSet();
		SelectorFrame sf = model.getSelectorFrame();
		if(row)
		    group = sf.getSelectedRowGroup();
		else
		    group = sf.getSelectedColumnGroup();


		if(algorithm instanceof NeedsISimilarity){
		    ISimilarity sim = model.getISimilarity(row);
		    
		    ((NeedsISimilarity)algorithm).setSimilarity(sim);

		    if(algorithm instanceof RelativeClusterer){
			sim = model.getISimilarity(!row);
			((NeedsISimilarity)algorithm).setSimilarity(sim);
			IObjectGrouping otherGrouping = 
			    model.getGUIIMatrix().getCurrentGrouping(!row);
			((NeedsOtherIObjectGrouping)algorithm).setOtherIObjectGrouping(otherGrouping);
		    }
		}
		if(algorithm instanceof NeedsSparseISimilarity){
		    ISimilarity sim = model.getISimilarity(row);
		    if(sim instanceof SparseISimilarity){
			SparseISimilarity ssim = 
			    (SparseISimilarity) sim;
			((NeedsSparseISimilarity)algorithm).setSimilarity(ssim);
			if(algorithm instanceof RelativeClusterer){
			    sim = model.getISimilarity(!row);
			    if(sim instanceof SparseISimilarity){
				ssim = (SparseISimilarity) sim;
				((NeedsSparseISimilarity)algorithm).setSimilarity(ssim);
			    }else{
				((NeedsSparseISimilarity)algorithm).setSimilarity(null);
			    }
			    IObjectGrouping otherGrouping = 
				model.getGUIIMatrix().getCurrentGrouping(!row);
			    ((NeedsOtherIObjectGrouping)algorithm).setOtherIObjectGrouping(otherGrouping);
			}
		    }
		}
		if(algorithm instanceof NeedsIMatrix){
		    ((NeedsIMatrix)algorithm).setIMatrix(model.getIMatrix());
		}
		if(algorithm instanceof NeedsIMatrixDimension){
		    ((NeedsIMatrixDimension)algorithm).setIMatrixDimension(row);
		}
		if(algorithm instanceof NeedsOtherIObjectGrouping){
		    IObjectGrouping otherGrouping = 
			model.getGUIIMatrix().getCurrentGrouping(!row);
		    ((NeedsOtherIObjectGrouping)algorithm).setOtherIObjectGrouping(otherGrouping);
		}

		//		IObjectGrouping clustering = algorithm.cluster(group);
		IObjectGrouping clustering = 
		    algorithm.cluster((IObjectGroup) group);
		if(clustering!=null){

		    clustering.removeEmptyGroups();
		    
		    GUIIMatrix guimatrix = model.getGUIIMatrix();
		    guimatrix.getGroupings(row).setGrouping(clustering);
		    view.getGroupingManager(row).putNewGrouping(clustering);

		    view.resetImage();
		    view.validateIMatrixBorders();
		    view.forceChange();
		    view.getOverview().forceChange();
		    
		    
		    model.update();
		    /*
		      model.setChanged();
		      model.notifyObservers();
		    */
		    
		}else{
		    System.out.println("No clustering created.");
		}

		// Ta bort allt frn algoritmen
		if(algorithm instanceof NeedsISimilarity){
		    ((NeedsISimilarity)algorithm).setSimilarity(null);
		}
		if(algorithm instanceof NeedsSparseISimilarity){
		    ((NeedsSparseISimilarity)algorithm).setSimilarity(null);
		}
		if(algorithm instanceof NeedsIMatrix){
		    ((NeedsIMatrix)algorithm).setIMatrix(null);
		}
		if(algorithm instanceof NeedsOtherIObjectGrouping){
		    ((NeedsOtherIObjectGrouping)algorithm).setOtherIObjectGrouping(null);
		}
		
		pf.dispose();
					
	    }else if(source == cancel){
		setVisible(false);
	    }
	}
	/*
	}else if(e.getSource() instanceof JComboBox){ //AlgorithmSelectionPanel){
	    if(algorithmSelectionPanel.actionLeadToChange()){
		algorithm = algorithmSelectionPanel.getSelectedAlgorithm();
		//		System.out.println("New Alg: "+algorithm.getAlgorithmName());
		String path = (String) model.getPropertyValue("Result Path");
		algPropertiesPanel = new PropertiesPanel(algorithm, path);
		putAll();
	    }
	}
	*/
    }
    
    public void setAlgorithm(GroupingAlgorithm algorithm){
	this.algorithm=algorithm;
	String path = (String) model.getPropertyValue("Result Path");
	algPropertiesPanel = new PropertiesPanel(algorithm, path);
	putAll();
    }


    public void paint(Graphics g){
	super.paint(g);
    }

		
}

