package infomat.view.algorithms;

import infomat.algorithms.*;
import infomat.io.*;
import infomat.io.textresult.*;
import infomat.vectorspace.*;
import infomat.view.*;
import infomat.view.grouping.settings.*;
import infomat.view.picture.*;
import mro.util.properties.*;
import mro.util.properties.Properties;
import mro.util.properties.dialog.*;
import mro.util.TracePrinter;

import java.awt.*;
import java.awt.Dialog.*;//ModalityType;
import java.awt.event.*;

import javax.swing.border.*;
import javax.swing.filechooser.FileFilter;

import java.util.*;
import java.io.*;

import javax.swing.*;



public class MatrixFilterFrame extends JFrame implements ActionListener{

    public Dimension MATRIXFILTERFRAME_PREFERRED_SIZE = 
	new Dimension(300, 500);

    //    private IMatrix matrix;
    private InfomatModel model;
    private InfomatView view;

    private JButton apply;
    private JButton cancel;

    private JFileChooser fileChooser;

    //    private Properties properties;
    private IMatrixCellFilter matrixCellFilter = new IMatrixCellFilter();

    private boolean printOn=true;
    private TracePrinter tp = new TracePrinter(printOn);

	
    public MatrixFilterFrame(InfomatModel model,
			     InfomatView view){
	super("Matrix Filter");
	//	this.matrix=matrix;
	this.model = model;
	this.view = view;

	if(model.getIMatrix()==null){
	    dispose();
	    return;
	}

	String fileChooserPath = (String) model.getPropertyValue("Matrix File Path");
	fileChooser = new JFileChooser(fileChooserPath);
	fileChooser.setFileFilter(new FileFilter() {
		public String getDescription() {
		    return "XML files";
		}

		public boolean accept(File f) {
		    if (f.isDirectory())
			return true;
		    int findresult = f.getName().indexOf(".xml");
		    int len = f.getName().length();
		    return findresult != -1 && findresult == len - 4;
		}

	    });

	Container container = getContentPane();
	container.setLayout(new BoxLayout(container, BoxLayout.Y_AXIS));


	//		apply = new JButton("Apply");
	//		JButton dummyButton = new JButton("Cancel");
	//		int height = (int) dummy.getPreferredSize().getHeight();
	//		int width = (int) dummyButton.getPreferredSize().getWidth();
	//		Dimension buttonSize = new Dimension(width, height);

	//		apply.addActionListener(this);
	//		add(apply);


	JTextField dummy = new JTextField("Apply");
	JButton dummyButton = new JButton("Cancel");
	int height = (int) dummy.getPreferredSize().getHeight();
	int width = (int) dummyButton.getPreferredSize().getWidth();
	Dimension buttonSize = new Dimension(width, height);

	JPanel infoAndButtons = new JPanel();
	infoAndButtons.setLayout(new GridLayout(2,1));
	JLabel info = new JLabel("Alter properties below and press \"Apply\"");
	infoAndButtons.add(info);


	JPanel buttonPanel = new JPanel();
	buttonPanel.setLayout(new BoxLayout(buttonPanel, BoxLayout.X_AXIS));
	apply = new JButton("Apply");
	apply.setPreferredSize(buttonSize);
	apply.addActionListener(this);
	buttonPanel.add(apply);
	cancel = new JButton("Cancel");
	cancel.setPreferredSize(buttonSize);
	cancel.addActionListener(this);
	buttonPanel.add(cancel);
	infoAndButtons.add(buttonPanel);
	infoAndButtons.setBorder(new TitledBorder("Filter Matrix"));
	container.add(infoAndButtons);


	PropertiesPanel pp = new PropertiesPanel(matrixCellFilter);
	container.add(pp);

	setVisible(true);
	setPreferredSize(MATRIXFILTERFRAME_PREFERRED_SIZE);
	pack();
    }

    public void actionPerformed(ActionEvent e) {
	if(e.getSource() instanceof JButton){
	    JButton source = (JButton) e.getSource();
	    if(source == apply){
		//				ProgressFrame pf = new ProgressFrame("Clustering - "+algorithm.getAlgorithmName());
		IMatrix matrix = model.getIMatrix();
		matrixCellFilter.filter(matrix);

		GUIIMatrix guimatrix = model.getGUIIMatrix();
		guimatrix.getRowGroupings().purgeGroupingsToGroup();
		guimatrix.getColumnGroupings().purgeGroupingsToGroup();

		System.out.println("Matrix: "+matrix.getSize());
		
		
		if(matrix.getAllCells().size()<0 ||
		   matrix.getRowIObjectSet().numberOfIObjects()==0 ||
		   matrix.getColumnIObjectSet().numberOfIObjects()==0){
		    model.resetIMatrix();
		    tp.println("MatrixFilterFrame: no cells in matrix!");

		}else{

		    IObjectGrouping grouping = 
			model.getGUIIMatrix().getCurrentColumnGrouping();
		    //		grouping.removeEmptyGroups();
		    view.getColumnGroupingManager().getColoringForGrouping(grouping).recolor();
		    view.validateIMatrixBorders();

		    view.forceChange();
		    view.getOverview().forceChange();
		}


		model.update();
			
	    }else if(source == cancel){
		dispose();
	    }
	}
    }
}
