package infomat.view.tools;

import infomat.view.*;
import infomat.view.grouping.settings.*;
import infomat.view.picture.*;
import infomat.view.vector.*;
import infomat.vectorspace.*;
import infomat.vectorspace.similarity.*;
import infomat.vectorspace.ivector.*;

import java.awt.*;
import java.awt.event.*;
import java.util.*;

import javax.swing.*;
import javax.swing.border.LineBorder;
import javax.swing.border.TitledBorder;


/**
 * Presents IObjects for rows and columns. Used by SearchFrame.
 *
 * @version          090727
 * @author           Magnus Rosell
 */
public class RowColumnPanel extends JPanel implements Observer{

    private InfomatModel model;
    private InfomatView view;

    private JTabbedPane pane = new JTabbedPane();
    private IVectorPanel rows;
    private IVectorPanel columns;

    private IObjectGrouping currentRowGrouping = null;
    private IObjectGrouping currentColumnGrouping = null;

    public RowColumnPanel(InfomatModel model, InfomatView view, boolean selectable, boolean fileable, boolean orderable){
	//	this.pixelFrame=pixelFrame;
	this.model=model;
	this.view = view;

	rows = new IVectorPanel(model, true, true);
	columns = new IVectorPanel(model, true, true);

		
	setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
	

	rows.setSelectable(selectable);
	columns.setSelectable(selectable);

	rows.setFileable(fileable);
	rows.setFilePath((String) model.getPropertyValue("Result Path"));
	columns.setFileable(fileable);
	columns.setFilePath((String) model.getPropertyValue("Result Path"));

	rows.setOrderable(orderable);
	columns.setOrderable(orderable);

	pane.add("Rows", rows);
	pane.add("Columns", columns);
		
	add(pane);

	model.addObserver(this);
	model.getSimilarityManager().addObserver(this);

	revalidate();
	repaint();

	resetGroupings();
    }

    private void resetGroupings(){

	if(model.getIMatrix()==null){
	    rows.setIVector(new IVector<IVectorIndex>());
	    columns.setIVector(new IVector<IVectorIndex>());
	    repaint();
	    return;
	}

	boolean row = true;
	rows.setSimilarity(model.getISimilarity(row));
	row = false;
	columns.setSimilarity(model.getISimilarity(row));


	boolean change = false;

	IObjectGrouping newRowGrouping = 
	    model.getGUIIMatrix().getRowGroupings().getCurrentGrouping();
	if(newRowGrouping != currentRowGrouping){
	    currentRowGrouping = newRowGrouping;
	    rows.setRestrictionGroup(currentRowGrouping);
	    rows.setIVector(new IVector<IVectorIndex>());
	    change = true;
	}

	IObjectGrouping newColumnGrouping = 
	    model.getGUIIMatrix().getColumnGroupings().getCurrentGrouping();
	if(newColumnGrouping != currentColumnGrouping){
	    currentColumnGrouping = newColumnGrouping;
	    columns.setRestrictionGroup(currentColumnGrouping);
	    columns.setIVector(new IVector<IVectorIndex>());
	    change = true;
	}

	if(change){
	    repaint();
	}
    }

    /** Called by the Observable */
    public void update(Observable o, Object arg) {
	if(o==model){
	    resetGroupings();
	}else if(o==model.getSimilarityManager()){
	    if(rows.getOrderable()){
		boolean row = true;
		rows.setSimilarity(model.getISimilarity(row));
	    }
	    if(columns.getOrderable()){
		boolean row = false;
		columns.setSimilarity(model.getISimilarity(row));
	    }

	    //	    resetGroupings();

	}
    }

    /*
    public void set(SparsePixel sp){
		
	if(sp!=null){
	    Vector<IMatrixCell> vec = sp.getAllIMatrixCells();
	    IVector<IVectorIndex> cellvec = new IVector<IVectorIndex>();
	    IVector<IVectorIndex> rowvec = new IVector<IVectorIndex>();
	    IVector<IVectorIndex> colvec = new IVector<IVectorIndex>();
	    for(int i=0; i<vec.size(); i++){
		IMatrixCell mc = vec.get(i);

		cellvec.put(mc, mc.getWeight());

		if(!rowvec.contains(mc.getRowIObject()))
		    rowvec.put(mc.getRowIObject(), mc.getWeight());
		else if(mc.getWeight() > rowvec.getValue(mc.getRowIObject()))
		    rowvec.put(mc.getRowIObject(), mc.getWeight());
		
		if(!colvec.contains(mc.getColumnIObject()))
		    colvec.put(mc.getColumnIObject(), mc.getWeight());
		else if(mc.getWeight() > colvec.getValue(mc.getColumnIObject()))
		    colvec.put(mc.getColumnIObject(), mc.getWeight());
	    }
	    cells.setIVector(cellvec);
	    rows.setIVector(rowvec);
	    columns.setIVector(colvec);

	}else{
	    IVector<IVectorIndex> ivec = new IVector<IVectorIndex>();
	    cells.setIVector(ivec);
	    rows.setIVector(ivec);
	    columns.setIVector(ivec);
	}
	repaint();
    }
    */



    /*	
    public void set(IObject rowObject, IObject colObject){
		
	SparsePicture sparsePicture = view.getSparsePicture();
	SparsePixel sp = sparsePicture.getPixelForObjects(rowObject, colObject);
	if(sp!=null){
	    Vector<IMatrixCell> vec = sp.getAllIMatrixCells();
	    IVector<IVectorIndex> cellvec = new IVector<IVectorIndex>();
	    IVector<IVectorIndex> rowvec = new IVector<IVectorIndex>();
	    IVector<IVectorIndex> colvec = new IVector<IVectorIndex>();
	    for(int i=0; i<vec.size(); i++){
		IMatrixCell mc = vec.get(i);
		cellvec.put(mc, mc.getWeight());

		if(!rowvec.contains(mc.getRowIObject()))
		    rowvec.put(mc.getRowIObject(), mc.getWeight());
		else if(mc.getWeight() > rowvec.getValue(mc.getRowIObject()))
		    rowvec.put(mc.getRowIObject(), mc.getWeight());
		
		if(!colvec.contains(mc.getColumnIObject()))
		    colvec.put(mc.getColumnIObject(), mc.getWeight());
		else if(mc.getWeight() > colvec.getValue(mc.getColumnIObject()))
		    colvec.put(mc.getColumnIObject(), mc.getWeight());
	    }
	    cells.setIVector(cellvec);
	    rows.setIVector(rowvec);
	    columns.setIVector(colvec);

	}else{
	    IVector<IVectorIndex> ivec = new IVector<IVectorIndex>();
	    cells.setIVector(ivec);
	    rows.setIVector(ivec);
	    columns.setIVector(ivec);
	}
	repaint();
    }
    */


    /*
    public void set(Vector matrixCells){
	if(matrixCells!=null){
	    IVector<IVectorIndex> cellvec = new IVector<IVectorIndex>();
	    IVector<IVectorIndex> rowvec = new IVector<IVectorIndex>();
	    IVector<IVectorIndex> colvec = new IVector<IVectorIndex>();
	    for(int i=0; i<matrixCells.size(); i++){
		IMatrixCell mc = (IMatrixCell) matrixCells.get(i);
		cellvec.put(mc, mc.getWeight());

		if(!rowvec.contains(mc.getRowIObject()))
		    rowvec.put(mc.getRowIObject(), mc.getWeight());
		else if(mc.getWeight() > rowvec.getValue(mc.getRowIObject()))
		    rowvec.put(mc.getRowIObject(), mc.getWeight());
		
		if(!colvec.contains(mc.getColumnIObject()))
		    colvec.put(mc.getColumnIObject(), mc.getWeight());
		else if(mc.getWeight() > colvec.getValue(mc.getColumnIObject()))
		    colvec.put(mc.getColumnIObject(), mc.getWeight());
	    }
	    cells.setIVector(cellvec);
	    rows.setIVector(rowvec);
	    columns.setIVector(colvec);

	}else{
	    IVector<IVectorIndex> ivec = new IVector<IVectorIndex>();
	    cells.setIVector(ivec);
	    rows.setIVector(ivec);
	    columns.setIVector(ivec);
	}
	repaint();

    }
    */

    //////
    /*
    public void setAllFrom(PixelPanel pixelPanel){
	IVector<IVectorIndex> cellVec = pixelPanel.cells.getIVector();
	cells.setIVector(cellVec);

	IVector<IVectorIndex> rowVec = pixelPanel.rows.getIVector();
	rows.setIVector(rowVec);

	IVector<IVectorIndex> columnsVec = pixelPanel.columns.getIVector();
	columns.setIVector(columnsVec);

	repaint();
    }

    public void setCellsFrom(PixelPanel pixelPanel){
	IVector<IVectorIndex> cellVec = pixelPanel.cells.getIVector();
	cells.setIVector(cellVec);
	repaint();
    }

    public void setRowsFrom(PixelPanel pixelPanel){
	IVector<IVectorIndex> rowVec = pixelPanel.rows.getIVector();
	rows.setIVector(rowVec);
	repaint();
    }

    public void setColumnsFrom(PixelPanel pixelPanel){
	IVector<IVectorIndex> columnsVec = pixelPanel.columns.getIVector();
	columns.setIVector(columnsVec);
	repaint();
    }
    */

    ///
    /*
    public void setRowsFromColumns(PixelPanel pixelPanel){
	IVector<IVectorIndex> repr = new IVector<IVectorIndex>();
	ISimilarity sim = model.getISimilarity(false);
	if(sim instanceof SparseISimilarity){
	    SparseISimilarity ssim = (SparseISimilarity) sim;

	    IVector<IVectorIndex> columnsVec = pixelPanel.columns.getIVector();

	    if(columnsVec.size()>1){
		IObjectGroup group = new IObjectGroup("tmp");
		Vector<IVectorIndex> vii = columnsVec.getUnorderedVector();
		for(int i=0; i<vii.size(); i++){
		    group.add((IObject) vii.get(i));
		}
		
		repr = ssim.getGroupRepresentation(group);
	    }else if(columnsVec.size()==1){
		repr = ssim.getRepresentation((IObject) columnsVec.getUnorderedVector().get(0));
	    }
	}
	rows.setIVector(repr);
	repaint();
    }

    public void setColumnsFromRows(PixelPanel pixelPanel){
	IVector<IVectorIndex> repr = new IVector<IVectorIndex>();
	ISimilarity sim = model.getISimilarity(true);
	if(sim instanceof SparseISimilarity){
	    SparseISimilarity ssim = (SparseISimilarity) sim;

	    IVector<IVectorIndex> rowsVec = pixelPanel.rows.getIVector();

	    if(rowsVec.size()>1){
		IObjectGroup group = new IObjectGroup("tmp");
		Vector<IVectorIndex> vii = rowsVec.getUnorderedVector();
		for(int i=0; i<vii.size(); i++){
		    group.add((IObject) vii.get(i));
		}
		
		repr = ssim.getGroupRepresentation(group);
	    }else if(rowsVec.size()==1){
		repr = ssim.getRepresentation((IObject) rowsVec.getUnorderedVector().get(0));
	    }
	}
	columns.setIVector(repr);
	repaint();
    }



    protected void setElements(Vector matrixElementVector){
	IVector<IVectorIndex> elementIVector = new IVector<IVectorIndex>();
	for(int i=0; i<matrixElementVector.size(); i++){
	    IMatrixCell mc = (IMatrixCell) matrixElementVector.get(i);
	    elementIVector.add(mc, mc.getWeight());
	}
	cells.setIVector(elementIVector);
	repaint();
    }
    */

    //////

    /*
    public void addAllFrom(PixelPanel pixelPanel){
	IVector<IVectorIndex> cellVec = cells.getIVector();
	cellVec.add(pixelPanel.cells.getIVector());
	cells.setIVector(cellVec);

	IVector<IVectorIndex> rowVec = rows.getIVector();
	rowVec.add(pixelPanel.rows.getIVector());
	rows.setIVector(rowVec);

	IVector<IVectorIndex> columnsVec = columns.getIVector();
	columnsVec.add(pixelPanel.columns.getIVector());
	columns.setIVector(columnsVec);

	repaint();
    }

    public void addCellsFrom(PixelPanel pixelPanel){
	IVector<IVectorIndex> cellVec = cells.getIVector();
	cellVec.add(pixelPanel.cells.getIVector());
	cells.setIVector(cellVec);
	repaint();
    }

    public void addRowsFrom(PixelPanel pixelPanel){
	IVector<IVectorIndex> rowVec = rows.getIVector();
	rowVec.add(pixelPanel.rows.getIVector());
	rows.setIVector(rowVec);
	repaint();
    }

    public void addColumnsFrom(PixelPanel pixelPanel){
	IVector<IVectorIndex> columnsVec = columns.getIVector();
	columnsVec.add(pixelPanel.columns.getIVector());
	columns.setIVector(columnsVec);
	repaint();
    }
    */


    //////
    /*
    public void putAllFrom(PixelPanel pixelPanel){
	IVector<IVectorIndex> cellVec = cells.getIVector();
	cellVec.put(pixelPanel.cells.getIVector());
	cells.setIVector(cellVec);

	IVector<IVectorIndex> rowVec = rows.getIVector();
	rowVec.put(pixelPanel.rows.getIVector());
	rows.setIVector(rowVec);

	IVector<IVectorIndex> columnsVec = columns.getIVector();
	columnsVec.put(pixelPanel.columns.getIVector());
	columns.setIVector(columnsVec);

	repaint();
    }

    public void putCellsFrom(PixelPanel pixelPanel){
	IVector<IVectorIndex> cellVec = cells.getIVector();
	cellVec.put(pixelPanel.cells.getIVector());
	cells.setIVector(cellVec);
	repaint();
    }

    public void putRowsFrom(PixelPanel pixelPanel){
	IVector<IVectorIndex> rowVec = rows.getIVector();
	rowVec.put(pixelPanel.rows.getIVector());
	rows.setIVector(rowVec);
	repaint();
    }

    public void putColumnsFrom(PixelPanel pixelPanel){
	IVector<IVectorIndex> columnsVec = columns.getIVector();
	columnsVec.put(pixelPanel.columns.getIVector());
	columns.setIVector(columnsVec);
	repaint();
    }
    */


    /*
    public void set(IObjectGroup group, boolean row){
		
	if(group!=null){
	    Vector<IObject> vec = group.getOrdered();
	    IVector<IVectorIndex> rowvec = new IVector<IVectorIndex>();
	    IVector<IVectorIndex> colvec = new IVector<IVectorIndex>();
	    for(int i=0; i<vec.size(); i++){
		IObject io = vec.get(i);

		if(row && !rowvec.contains(io))
		    rowvec.put(io, (float) group.getIObjectOrderIndex(io));

		if(!row && !colvec.contains(io))
		    colvec.put(io, (float) group.getIObjectOrderIndex(io));
	    }
	    rows.setIVector(rowvec);
	    columns.setIVector(colvec);

	}else{
	    IVector<IVectorIndex> ivec = new IVector<IVectorIndex>();
	    rows.setIVector(ivec);
	    columns.setIVector(ivec);
	}

	IVector<IVectorIndex> ivec = new IVector<IVectorIndex>();
	cells.setIVector(ivec);

	repaint();
    }
    */

    protected IVector getColumns(){
	return columns.getIVector();
    }

    protected IVector getRows(){
	return rows.getIVector();
    }



    protected void addColumns(IVector<IObject> vec){
	columns.addIVector(vec);
    }

    protected void addRows(IVector<IObject> vec){
	rows.addIVector(vec);
    }

    protected void setColumns(IVector<IObject> vec){
	columns.setIVector(vec);
    }

    protected void setRows(IVector<IObject> vec){
	rows.setIVector(vec);
    }



    public void paintComponent(Graphics g){
	super.paintComponent(g);
    }
	

 
}
