package infomat.view.tools;

import infomat.algorithms.grouping.*;
import infomat.algorithms.search.*;
import infomat.io.*;
import infomat.vectorspace.*;
import infomat.vectorspace.ivector.*;
import infomat.vectorspace.similarity.*;
import infomat.view.*;
import infomat.view.grouping.settings.*;
import infomat.view.util.*;
import infomat.view.vector.*;
import infomat.view.selector.*;
import mro.util.properties.*;
import mro.util.properties.Properties;
import mro.util.properties.dialog.*;

import java.awt.*;
import java.awt.Dialog.*;//ModalityType;
import java.awt.event.*;
import javax.swing.border.*;
import javax.swing.text.*;
import java.util.*;
import java.io.*;

import java.util.regex.Pattern;

import javax.swing.*;
import javax.swing.filechooser.FileFilter;


	
/**
 * A search tool.
 *
 * @version          090727
 * @author           Magnus Rosell
 */
public class SearchFrame extends JFrame
    implements ActionListener{

    private static int searchNumber = 1;

    public Dimension SEARCHPANELS_PREFERRED_SIZE = 
	new Dimension(350, 600);

    public Dimension SEARCHFRAME_PREFERRED_SIZE = 
	new Dimension(1050, 600);

    private InfomatModel model;
    private InfomatView view;

    private JPanel framePanel;


    private RowColumnPanel searchPanel;
    private JTextField searchTextField;
    private SmallButton addRowsB;
    private SmallButton addColsB;
    private SmallButton setRowsB;
    private SmallButton setColsB;

    private RowColumnPanel resultPanel;
    private SmallButton searchB;

    
    private Searcher searcher = new Searcher();
    private PropertiesPanel propertiesPanel;
    private String[] listOrTextStrings = {"List", "Text"};
    private JComboBox listOrTextBox = new JComboBox(listOrTextStrings);




    private JPanel usePanel;
    private SmallButton groupingApplyB = new SmallButton("Apply");;

    private String[] rowOrColumnStrings = {"Rows", "Columns"};
    private JComboBox rowOrColumnBox = new JComboBox(rowOrColumnStrings);

    private String[] queryOrResultStrings = {"Result", "Query List"};
    private JComboBox queryOrResultBox = new JComboBox(queryOrResultStrings);

    private String[] oneOrTwoStrings = {"One Cluster", "Two Clusters"};
    private JComboBox oneOrTwoBox = new JComboBox(oneOrTwoStrings);

    private String[] shownOrColorStrings = {"Shown", "Color"};
    private JComboBox shownOrColorBox = new JComboBox(shownOrColorStrings);



		
    private AlternatingButton rowsOrColumns;
    //    private SmallButton rowsOrColumns;
    private SmallButton apply;
    private SmallButton cancel;

    private SmallButton load;
    private SmallButton save;
    private SmallButton fromIOToStrings;

    private SmallButton fromStringsToIO;
    private SmallButton stoplistToIO;


    IVectorPanel<IVectorString> stringsPanel;
    IVectorPanel<IObject> iobjectsPanel;

    private JFileChooser fileChooser;
    private String fileChooserPath="";

    public SearchFrame(InfomatModel model, InfomatView view){
	super("Search");

	this.model=model;
	this.view=view;
			
	if(view==null || model==null || model.getIMatrix() == null){
	    dispose();
	    return;
	}


	setFileChooser();

	setVisible(true);
	setPreferredSize(SEARCHFRAME_PREFERRED_SIZE);

	framePanel = new JPanel();
	framePanel.setPreferredSize(SEARCHFRAME_PREFERRED_SIZE);

	add(framePanel);
	putAll();
	pack();
    }


    private void setFileChooser(){
	fileChooserPath = (String) model.getPropertyValue("Result Path");
	fileChooser = new JFileChooser(fileChooserPath);
	fileChooser.setFileFilter(new FileFilter() {
		public String getDescription() {
		    return "txt files";
		}

		public boolean accept(File f) {
		    if (f.isDirectory())
			return true;
		    int findresult = f.getName().indexOf(".txt");
		    int len = f.getName().length();
		    return findresult != -1 && findresult == len - 4;
		}

	    });
    }



    public void putAll(){
	//	removeAll();
	framePanel.removeAll();
	
	Container container = getContentPane();
	framePanel.setLayout(new BoxLayout(framePanel, BoxLayout.X_AXIS));

	JPanel leftPanel = constructLeftPanel();
	framePanel.add(leftPanel);

	JPanel middlePanel = constructMiddlePanel();
	framePanel.add(middlePanel);


	JPanel rightPanel = constructRightPanel();
	framePanel.add(rightPanel);

	framePanel.revalidate();
	framePanel.repaint();
	repaint();
    }


    private JPanel constructLeftPanel(){
	JPanel leftPanel = new JPanel();
	leftPanel.setPreferredSize(SEARCHPANELS_PREFERRED_SIZE);
	leftPanel.setLayout(new BoxLayout(leftPanel, BoxLayout.Y_AXIS));

	searchTextField = new CutAndPasteJTextField();

	JTextField dummy = new JTextField("Set Rows   Set Columns                          jjyg");
	/*
	int height = (int) (2 * dummy.getPreferredSize().getHeight());
	int width = (int) (SEARCHPANELS_PREFERRED_SIZE.getWidth() - 10);
	Dimension dim = new Dimension(width, height);
	*/
	int height = (int) (2 * dummy.getPreferredSize().getHeight());
	int width = (int) dummy.getPreferredSize().getWidth();
	Dimension dim = new Dimension(width, height);

	searchTextField.setPreferredSize(dim);
	searchTextField.setMaximumSize(dim);
	searchTextField.setMinimumSize(dim);

	searchTextField.addActionListener(this);
	leftPanel.add(searchTextField);

	JPanel buttonPanel = new JPanel();
	buttonPanel.setLayout(new BoxLayout(buttonPanel, BoxLayout.Y_AXIS));

	JPanel addPanel = new JPanel();
	addPanel.setLayout(new BoxLayout(addPanel, BoxLayout.X_AXIS));
	addRowsB = new SmallButton("Add Rows");
	addRowsB.addActionListener(this);
	addPanel.add(addRowsB);
	addColsB = new SmallButton("Add Columns");
	addPanel.add(addColsB);
	addColsB.addActionListener(this);
	buttonPanel.add(addPanel);

	JPanel setPanel = new JPanel();
	setPanel.setLayout(new BoxLayout(setPanel, BoxLayout.X_AXIS));
	setRowsB = new SmallButton("Set Rows");
	setPanel.add(setRowsB);
	setRowsB.addActionListener(this);
	setColsB = new SmallButton("Set Columns");
	setPanel.add(setColsB);
	setColsB.addActionListener(this);
	buttonPanel.add(setPanel);

	leftPanel.add(buttonPanel);
	
	boolean selectable = true;
	boolean fileable = true;
	boolean orderable = true;
	searchPanel=new RowColumnPanel(model, view, selectable, fileable, orderable);

	leftPanel.add(searchPanel);

	leftPanel.setBorder(new TitledBorder("Query"));
	//	leftPanel.setAlignmentX(LEFT_ALIGNMENT);


	return leftPanel;
    }


    private JPanel constructMiddlePanel(){


	JPanel middlePanel = new JPanel();
	middlePanel.setPreferredSize(SEARCHPANELS_PREFERRED_SIZE);
	middlePanel.setLayout(new BoxLayout(middlePanel, BoxLayout.Y_AXIS));

	JPanel searchPanel = new JPanel();

	Dimension dim = listOrTextBox.getPreferredSize();
	listOrTextBox.setPreferredSize(dim);
	listOrTextBox.setMaximumSize(dim);
	listOrTextBox.setMinimumSize(dim);
	queryOrResultBox.setPreferredSize(dim);
	searchPanel.add(listOrTextBox);


	searchB = new SmallButton("Search");
	searchB.addActionListener(this);
	searchPanel.add(searchB);


       	searchPanel.setBorder(new TitledBorder("Search"));
	middlePanel.add(searchPanel);
	searchPanel.setAlignmentX(LEFT_ALIGNMENT);

	propertiesPanel = new PropertiesPanel(searcher);
	middlePanel.add(propertiesPanel);
	propertiesPanel.setAlignmentX(LEFT_ALIGNMENT);


	/*
	int width = (int) searchPanel.getPreferredSize().getWidth();
	int height = (int) (2 * searchB.getPreferredSize().getHeight());
	Dimension dim = new Dimension(width, height);
	searchPanel.setPreferredSize(dim);
	searchPanel.setMaximumSize(dim);
	searchPanel.setMinimumSize(dim);
	*/



	/*
	JTextField dummy = new JTextField("Dummy");
	int height = (int) (dummy.getPreferredSize().getHeight());
	int width = (int) (SEARCHPANELS_PREFERRED_SIZE.getWidth() - 10);
	Dimension dim = new Dimension(width, height);
	*/
	/*
	JTextField dummy = new JTextField("Tjo Clustejs");
	Dimension dim = dummy.getPreferredSize();
	*/

	JPanel groupingPanel = new JPanel();
	groupingPanel.setLayout(new BoxLayout(groupingPanel, BoxLayout.Y_AXIS));
	groupingPanel.setAlignmentX(LEFT_ALIGNMENT);

	dim = oneOrTwoBox.getPreferredSize();
	rowOrColumnBox.setPreferredSize(dim);
	rowOrColumnBox.setMaximumSize(dim);
	rowOrColumnBox.setMinimumSize(dim);
	queryOrResultBox.setPreferredSize(dim);
	queryOrResultBox.setMaximumSize(dim);
	queryOrResultBox.setMinimumSize(dim);
	oneOrTwoBox.setPreferredSize(dim);
	oneOrTwoBox.setMaximumSize(dim);
	oneOrTwoBox.setMinimumSize(dim);
	shownOrColorBox.setPreferredSize(dim);
	shownOrColorBox.setMaximumSize(dim);
	shownOrColorBox.setMinimumSize(dim);
	
	groupingPanel.add(rowOrColumnBox);
	groupingPanel.add(queryOrResultBox);
	groupingPanel.add(oneOrTwoBox);
	groupingPanel.add(shownOrColorBox);

	rowOrColumnBox.setAlignmentX(LEFT_ALIGNMENT);
	queryOrResultBox.setAlignmentX(LEFT_ALIGNMENT);
	oneOrTwoBox.setAlignmentX(LEFT_ALIGNMENT);
	shownOrColorBox.setAlignmentX(LEFT_ALIGNMENT);

	
	groupingPanel.add(groupingApplyB);
	groupingApplyB.setAlignmentX(LEFT_ALIGNMENT);
	groupingApplyB.addActionListener(this);

	groupingPanel.setBorder(new TitledBorder("Grouping"));

	middlePanel.add(groupingPanel);
	groupingPanel.setAlignmentX(LEFT_ALIGNMENT);



	return middlePanel;
    }




    private JPanel constructRightPanel(){
	JPanel rightPanel = new JPanel();
	rightPanel.setPreferredSize(SEARCHPANELS_PREFERRED_SIZE);
	rightPanel.setLayout(new BoxLayout(rightPanel, BoxLayout.Y_AXIS));
	rightPanel.setAlignmentX(LEFT_ALIGNMENT);

	boolean selectable = true;
	boolean fileable = true;
	boolean orderable = true;
	resultPanel=new RowColumnPanel(model, view, selectable, fileable, orderable);

	//	resultContainerPanel.add(resultPanel);
	//	resultContainerPanel.setBorder(new TitledBorder("Result"));

	//	resultPanel.setBorder(new TitledBorder("Result"));
	resultPanel.setBorder(new TitledBorder("Result"));

	rightPanel.add(resultPanel);
	rightPanel.add(Box.createVerticalGlue());
	//	middlePanel.setAlignmentX(LEFT_ALIGNMENT);


	//	middlePanel.setAlignmentX(LEFT_ALIGNMENT);
	return rightPanel;
    }

 
    public void actionPerformed(ActionEvent e) {
	if(e.getSource() instanceof JButton){
	    JButton source = (JButton) e.getSource();
	    if(source == searchB){

		search(true);
		search(false);

	    }else if(source == addRowsB){
		//		IVector<IObject> ivec = parseText(searchTextField.getText(), true);
		IVector<IObject> ivec = searcher.parseText(searchTextField.getText(), model.getIMatrix(), model.getSelectorFrame().getSelectedRowGroup(), model.getSelectorFrame().getSelectedColumnGroup(), false);
		searchPanel.addRows(ivec);
	    }else if(source == addColsB){
		//		IVector<IObject> ivec = parseText(searchTextField.getText(), false);
		IVector<IObject> ivec = searcher.parseText(searchTextField.getText(), model.getIMatrix(), model.getSelectorFrame().getSelectedRowGroup(), model.getSelectorFrame().getSelectedColumnGroup(), true);
		searchPanel.addColumns(ivec);

	    }else if(source == setRowsB){
		//		IVector<IObject> ivec = parseText(searchTextField.getText(), true);
		IVector<IObject> ivec = searcher.parseText(searchTextField.getText(), model.getIMatrix(), model.getSelectorFrame().getSelectedRowGroup(), model.getSelectorFrame().getSelectedColumnGroup(), false);
		searchPanel.setRows(ivec);

	    }else if(source == setColsB){
		//		IVector<IObject> ivecOld = parseText(searchTextField.getText(), false);

		IVector<IObject> ivec = searcher.parseText(searchTextField.getText(), model.getIMatrix(), model.getSelectorFrame().getSelectedRowGroup(), model.getSelectorFrame().getSelectedColumnGroup(), true);
		searchPanel.setColumns(ivec);

	    }else if(source == groupingApplyB){

		boolean row = true;
		if(((String) rowOrColumnBox.getSelectedItem()).equals("Columns"))
		    row = false;

		boolean query = true;
		if(((String) queryOrResultBox.getSelectedItem()).equals("Result"))
		    query = false;

		boolean one = true;
		if(((String) oneOrTwoBox.getSelectedItem()).equals("Two Clusters"))
		    one = false;

		boolean shown = true;
		if(((String) shownOrColorBox.getSelectedItem()).equals("Color"))
		    shown = false;

		

		IVector<IObject> ivec = resultPanel.getColumns();
		if(row)
		    ivec = resultPanel.getRows();
		if(query){
		    ivec = searchPanel.getColumns();
		    if(row)
			ivec = searchPanel.getRows();
		}

		IObjectGrouping ioging = new IObjectGrouping("Search Grouping "+searchNumber);
		searchNumber++;
		IObjectGroup group = new IObjectGroup("Search Group");
		group.addAll(ivec);
		System.out.println("ivec: "+ivec.size());
		System.out.println("group: "+group.numberOfIObjects());

		ioging.add(group);

		if(!one){
		    SelectorFrame sf = model.getSelectorFrame();
		    IObjectGroup allGroup = sf.getSelectedRowGroup();
		    if(!row)
			allGroup = sf.getSelectedColumnGroup();
		    IObjectGroup rest = new IObjectGroup("Rest Group");

		    rest.addAll(allGroup.getIVector());
		    rest.remove(ivec);

		    ioging.add(rest);
		}


		//		ioging.removeEmptyGroups();
		    
		GUIIMatrix guimatrix = model.getGUIIMatrix();
		if(shown){
		    guimatrix.getGroupings(row).setGrouping(ioging);
		    view.getGroupingManager(row).putNewGrouping(ioging);
		}else{
		    guimatrix.getGroupings(row).addGrouping(ioging);
		    view.getGroupingManager(row).putNewGroupingAsColoring(ioging);
		}
		

		
		view.resetImage();
		view.validateIMatrixBorders();
		view.forceChange();
		view.getOverview().forceChange();
		
		
		model.update();


	    }





	}
    }


    private void search(boolean row){

	String searchMethod = (String) listOrTextBox.getSelectedItem();

	
	ISimilarity sim = model.getISimilarity(row);
	searcher.setSimilarity(sim);
	SelectorFrame sf = model.getSelectorFrame();

	IVector<IObject> resIVec = null;
	if(searchMethod.equals("List")){
	    IVector<IObject> queryVec = searchPanel.getColumns();
	    if(!row)
		queryVec = searchPanel.getRows();

	    resIVec = searcher.search(queryVec, model.getIMatrix(), sf.getSelectedRowGroup(), sf.getSelectedColumnGroup(), row);
	}else if(searchMethod.equals("Text")){
	    String searchString = searchTextField.getText();
	    
	    resIVec = searcher.search(searchString, model.getIMatrix(), sf.getSelectedRowGroup(), sf.getSelectedColumnGroup(), row);

	}


	if(row)
	    resultPanel.setRows(resIVec);
	else
	    resultPanel.setColumns(resIVec);

	searcher.setSimilarity(null);


	/*
	IMatrix matrix = model.getIMatrix();
	SelectorFrame sf = model.getSelectorFrame();

	IObjectGroup group = sf.getSelectedRowGroup();
	if(!row)
	    group = sf.getSelectedColumnGroup();
		
	IVector<IObject> queryVec = searchPanel.getColumns();
	if(!row)
	    queryVec = searchPanel.getRows();

		
	Vector<IObject> resVec = new Vector<IObject>();
	Vector<IObject> cv = queryVec.getUnorderedVector();
	for(int i=0; i<cv.size(); i++){
	    IObject io = cv.get(i);
	    Vector cells = matrix.getCellsWithIObjectRestrictedToGroup(io, group);
	    for(int j=0; j<cells.size(); j++){
		IMatrixCell mc = (IMatrixCell) cells.get(j);
		IObject io2 = mc.getOtherIObject(io);
		resVec.add(io2);
	    }
	}

	ISimilarity sim = model.getISimilarity(row);
	if(sim instanceof SparseISimilarity){

	    SparseISimilarity ssim = (SparseISimilarity) sim;
	    IVector<IObject> resIVec = new IVector<IObject>();
	    for(int i=0; i<resVec.size(); i++){
		IObject io = resVec.get(i);
			
		float value = ssim.sim(queryVec, ssim.getRepresentation(io));
		resIVec.add(io, value);
	    }
		    
	    if(row)
		resultPanel.setRows(resIVec);
	    else
		resultPanel.setColumns(resIVec);
		    
	}
	*/

    }

    /*
    private IVector<IObject> parseText(String s, boolean row){

	IObjectGroup group = model.getIMatrix().getColumnIObjectSet();
	SelectorFrame sf = model.getSelectorFrame();
	if(row)
	    group = sf.getSelectedRowGroup();
	else
	    group = sf.getSelectedColumnGroup();

	HashMap<String, IObject> stringObjectMap = 
	    new HashMap<String, IObject>();

	Vector<IObject> ordered = group.getOrdered();
	for(int i=0; i<ordered.size(); i++){
	    IObject io = ordered.get(i);
	    stringObjectMap.put(io.getString(), io);
	}

	IVector<IObject> ret = new IVector<IObject>();
	String[] parts = s.split(" ");
	for(int i=0; i<parts.length; i++){
	    IObject io = stringObjectMap.get(parts[i]);
	    if(io!=null)
		ret.add(io, (float) 1);
	    else if(parts[i].startsWith("[")){
		for(int j=0; j<ordered.size(); j++){
		    io = ordered.get(j);
		    if(Pattern.matches(parts[i], io.getString()))
		       ret.add(io, (float) 1);
		}
	    }
	    //	    System.out.println(parts[i]);
	}

	return ret;
    }
    */

    /*	
      private Vector<String> parseText(String s){
      Vector<String> ret = null;
      String[] parts = s.split(" ");
      if(parts.length>0){
      ret = new Vector<String>();
      for(int i=0; i<parts.length; i++){
      ret.add(parts[i]);
      System.out.println(parts[i]);
      }
      }

      return ret;
      }
    */
	


    private void showMessage(String title, String message){
	JOptionPane.showMessageDialog(
				      this, message, title, 
				      JOptionPane.INFORMATION_MESSAGE);

    }




    public void paint(Graphics g){
	super.paint(g);
    }


    private class CutAndPasteJTextField extends JTextField
	implements ActionListener, MouseListener{

	public JPopupMenu popup;

	public CutAndPasteJTextField(){
	    super();
	    initThings();
	}

	public CutAndPasteJTextField(Document doc, String text, int columns){
	    super(doc, text, columns);
	    initThings();
	}

	public CutAndPasteJTextField(int columns){
	    super(columns);
	    initThings();
	}

	public CutAndPasteJTextField(String text){
	    super(text);
	    initThings();
	}

	public CutAndPasteJTextField(String text, int columns){
	    super(text, columns);
	    initThings();
	}


	private void initThings(){
	    addMouseListener(this);
	    createPopupMenu();
	}


	/*
	private void createPopupMenu(JTextComponent text){
	    cut = new CutAction("Cut",null);
	    copy = new CopyAction("Copy", null);
	    paste = new PasteAction("Paste",null);
	    selectAll = new SelectAllAction("Select All",null);
	    cut.setTextComponent(text);
	    copy.setTextComponent(text);
	    paste.setTextComponent(text);
	    selectAll.setTextComponent(text);
	    
	    popup = new JPopupMenu();
	    popup.add( cut );
	    popup.add( copy );
	    popup.add( paste );
	    popup.addSeparator();
	    popup.add( selectAll );
	}
	*/

	private void createPopupMenu(){
	    //	    JPanel panel = new JPanel();

	    /*
	    cut = new CutAction("Cut",null);
	    copy = new CopyAction("Copy", null);
	    paste = new PasteAction("Paste",null);
	    selectAll = new SelectAllAction("Select All",null);
	    cut.setTextComponent(text);
	    copy.setTextComponent(text);
	    paste.setTextComponent(text);
	    selectAll.setTextComponent(text);
	    */

	    popup = new JPopupMenu();

	    /*
	    popup.add( cut );
	    popup.add( copy );
	    popup.add( paste );
	    popup.addSeparator();
	    popup.add( selectAll );
	    */

	    JMenuItem menuItem = new JMenuItem("Copy", KeyEvent.VK_O);
	    menuItem.addActionListener(this);
	    popup.add(menuItem);

	    menuItem = new JMenuItem("Paste", KeyEvent.VK_O);
	    menuItem.addActionListener(this);
	    popup.add(menuItem);

	    menuItem = new JMenuItem("Cut", KeyEvent.VK_O);
	    menuItem.addActionListener(this);
	    popup.add(menuItem);


	    //	    panel.add(popup);

	    //	    return panel;
	}


	/*** ActionListener ***/

	public void actionPerformed(ActionEvent e) {
	    if(e.getSource() instanceof JMenuItem){
		JMenuItem source = (JMenuItem) (e.getSource());
		String cmd = source.getText();
		/*
		String s = "Menu action event detected." + "    Event source: "
		    + source.getText();
		tp.println(s);
		*/

		/************************************************/
		// File menu
		if (cmd.equals("Cut")) {
		    cut();
		    System.out.println("Cut");
		} else if (cmd.equals("Paste")){
		    System.out.println("Paste");
		    paste();
		} else if (cmd.equals("Copy")){
		    System.out.println("Copy");
		    copy();
		}
	    }

	}
	
	/*** MouseListener ***/
	
	public void mouseClicked(MouseEvent e){
	    //          Invoked when the mouse button has been clicked (pressed and released) on a component.
	    checkPopup(e);
	}

	public void mouseEntered(MouseEvent e){
	    //          Invoked when the mouse enters a component.
	}
	public void mouseExited(MouseEvent e){
	    //          Invoked when the mouse exits a component.
	}
	public void mousePressed(MouseEvent e){
	    //          Invoked when a mouse button has been pressed on a component.
	    checkPopup(e);
	}

	public void mouseReleased(MouseEvent e){
	    //          Invoked when a mouse button has been released on a component.
	    checkPopup(e);
	}


	private void checkPopup(MouseEvent e) {
	    if (e.isPopupTrigger()) {
		popup.show(this, e.getX(), e.getY());
	    }
	}



    }


}

