package infomat.view.tools;

import infomat.algorithms.grouping.*;
import infomat.io.*;
import infomat.vectorspace.*;
import infomat.vectorspace.ivector.*;
import infomat.vectorspace.similarity.*;
import infomat.view.*;
import infomat.view.grouping.settings.*;
import infomat.view.util.*;
import infomat.view.vector.*;
import mro.util.properties.*;
import mro.util.properties.Properties;
import mro.util.properties.dialog.*;

import java.awt.*;
import java.awt.Dialog.*;//ModalityType;
import java.awt.event.*;
import javax.swing.border.*;
import java.util.*;
import java.io.*;

import javax.swing.*;
import javax.swing.filechooser.FileFilter;


	
/**
 * A stoplist tool.
 *
 * @version          080721
 * @author           Magnus Rosell
 */
public class StoplistFrame extends JFrame
    implements ActionListener{

    public Dimension STOPLISTPANELS_PREFERRED_SIZE = 
	new Dimension(350, 600);

    public Dimension STOPLISTFRAME_PREFERRED_SIZE = 
	new Dimension(1050, 600);

    private Stoplist stoplist;
    private JPanel stoplistPanel;
    private PropertiesPanel propertiesPanel;
    private InfomatModel model;
    private InfomatView view;
		
		
    private AlternatingButton rowsOrColumns;
    //    private SmallButton rowsOrColumns;
    private SmallButton apply;
    private SmallButton cancel;

    private SmallButton load;
    private SmallButton save;
    private SmallButton fromIOToStrings;

    private SmallButton fromStringsToIO;
    private SmallButton stoplistToIO;


    IVectorPanel<IVectorString> stringsPanel;
    IVectorPanel<IObject> iobjectsPanel;

    private JFileChooser fileChooser;
    private String fileChooserPath="";

    public StoplistFrame(InfomatModel model, InfomatView view){
	super("Stoplist");//algorithm.getAlgorithmName());

	this.model=model;
	this.view=view;
			
	if(view==null || model==null || model.getIMatrix() == null){
	    dispose();
	    return;
	}

	String[] alternatives = {"Columns", "Rows"};
	rowsOrColumns = new AlternatingButton(alternatives);
	//	rowsOrColumns = new SmallButton("Rows");

	setFileChooser();

	setVisible(true);
	setPreferredSize(STOPLISTFRAME_PREFERRED_SIZE);

	stoplist = new Stoplist(); //"/afs/nada.kth.se/home/f/u1ya3frf/Ordlistor/stoppord/Stoplist.txt");

	stoplistPanel = new JPanel();
	stoplistPanel.setPreferredSize(STOPLISTFRAME_PREFERRED_SIZE);

	add(stoplistPanel);
	putAll();
	pack();
    }


    private void setFileChooser(){
	fileChooserPath = (String) model.getPropertyValue("Result Path");
	fileChooser = new JFileChooser(fileChooserPath);
	fileChooser.setFileFilter(new FileFilter() {
		public String getDescription() {
		    return "txt files";
		}

		public boolean accept(File f) {
		    if (f.isDirectory())
			return true;
		    int findresult = f.getName().indexOf(".txt");
		    int len = f.getName().length();
		    return findresult != -1 && findresult == len - 4;
		}

	    });
    }



    public void putAll(){
	//	removeAll();
	stoplistPanel.removeAll();
	
	Container container = getContentPane();
	stoplistPanel.setLayout(new BoxLayout(stoplistPanel, BoxLayout.X_AXIS));

	JPanel leftPanel = constructLeftPanel();
	stoplistPanel.add(leftPanel);
			
	JPanel middlePanel = constructMiddlePanel();
	stoplistPanel.add(middlePanel);
			
	JPanel rightPanel = constructRightPanel();
	stoplistPanel.add(rightPanel);
			
			


	stoplistPanel.revalidate();
	stoplistPanel.repaint();
	repaint();
    }

    
    private JPanel constructRightPanel(){
	JPanel rightPanel = new JPanel();
	rightPanel.setPreferredSize(STOPLISTPANELS_PREFERRED_SIZE);
	rightPanel.setLayout(new BoxLayout(rightPanel, BoxLayout.Y_AXIS));

	JPanel iobjectsAndConvertPanel = new JPanel();
	iobjectsAndConvertPanel.setLayout(new BoxLayout(iobjectsAndConvertPanel, BoxLayout.Y_AXIS));

	JPanel convertButtonPanel = new JPanel();
	convertButtonPanel.setLayout(new FlowLayout(FlowLayout.LEFT));
	fromStringsToIO = new SmallButton("From Strings to IO");
	stoplistToIO = new SmallButton("Whole Stoplist to IO");
	//	fromStringsToIO.setPreferredSize(buttonSize);
	fromStringsToIO.addActionListener(this);
	stoplistToIO.addActionListener(this);
	convertButtonPanel.add(fromStringsToIO);
	convertButtonPanel.add(stoplistToIO);
	convertButtonPanel.setAlignmentX(LEFT_ALIGNMENT);
	iobjectsAndConvertPanel.add(convertButtonPanel);
	convertButtonPanel.setMinimumSize(convertButtonPanel.getPreferredSize());
	convertButtonPanel.setMaximumSize(convertButtonPanel.getPreferredSize());


	IMatrix matrix = model.getIMatrix();

	iobjectsPanel = new IVectorPanel<IObject>(model, true, true);
	iobjectsPanel.setFilePath((String) model.getPropertyValue("Result Path"));
	iobjectsPanel.setOrderable(true);
		boolean row = row();
		IObjectSet set = matrix.getColumnIObjectSet();
		if(row)
		    set = matrix.getRowIObjectSet();
		iobjectsPanel.setRestrictionGroup(set);

		iobjectsPanel.setIVector(stoplist.getStopIObjectsVector());
	iobjectsAndConvertPanel.add(iobjectsPanel);
	iobjectsPanel.setBorder(new TitledBorder("List"));
	iobjectsAndConvertPanel.setBorder(new TitledBorder("IObjects")); 
	rightPanel.add(iobjectsAndConvertPanel);

	//	rightPanel.add(Box.createVerticalGlue());



	JPanel main = new JPanel();
	main.setLayout(new BoxLayout(main, BoxLayout.Y_AXIS));
			
	JPanel dimPanel = new JPanel();
	dimPanel.setLayout(new BoxLayout(dimPanel, BoxLayout.X_AXIS));
	JLabel dimInfo = new JLabel("Choose rows or columns: ");
	dimPanel.add(dimInfo);
	rowsOrColumns.addActionListener(this);
	dimPanel.add(rowsOrColumns);
	main.add(dimPanel);
	dimPanel.setAlignmentX(LEFT_ALIGNMENT);
			
	JPanel buttonPanel = new JPanel();
	buttonPanel.setLayout(new BoxLayout(buttonPanel, BoxLayout.X_AXIS));
	apply = new SmallButton("Apply");
	//	apply.setPreferredSize(buttonSize);
	apply.addActionListener(this);
	buttonPanel.add(apply);
	cancel = new SmallButton("Cancel");
	//	cancel.setPreferredSize(buttonSize);
	cancel.addActionListener(this);
	buttonPanel.add(cancel);
	main.add(buttonPanel);
	buttonPanel.setAlignmentX(LEFT_ALIGNMENT);


	main.setBorder(new TitledBorder("Main")); //algorithm.getAlgorithmName()));
	main.setAlignmentX(LEFT_ALIGNMENT);
	rightPanel.add(main);

	int width = (int) STOPLISTPANELS_PREFERRED_SIZE.getWidth();
	int height = (int) main.getPreferredSize().getHeight();
	Dimension d = 
	    new Dimension(width, height);
			  
	main.setPreferredSize(d);
	main.setMinimumSize(main.getPreferredSize());
	main.setMaximumSize(main.getPreferredSize());

	/*			
	main.setMinimumSize(main.getPreferredSize());
	main.setMaximumSize(main.getPreferredSize());
	*/

	return rightPanel;
    }
			
    private JPanel constructMiddlePanel(){


	JPanel middlePanel = new JPanel();
	middlePanel.setPreferredSize(STOPLISTPANELS_PREFERRED_SIZE);
	middlePanel.setLayout(new BoxLayout(middlePanel, BoxLayout.Y_AXIS));


	JPanel convertButtonPanel = new JPanel();
	convertButtonPanel.setLayout(new FlowLayout(FlowLayout.RIGHT));
	fromIOToStrings = new SmallButton("From IO to Strings");
	fromIOToStrings.addActionListener(this);
	convertButtonPanel.add(fromIOToStrings);
	convertButtonPanel.setAlignmentX(LEFT_ALIGNMENT);

	middlePanel.add(convertButtonPanel);
	convertButtonPanel.setMinimumSize(convertButtonPanel.getPreferredSize());
	convertButtonPanel.setMaximumSize(convertButtonPanel.getPreferredSize());

	stringsPanel = new IVectorPanel<IVectorString>(model, true, true);
	stringsPanel.setFilePath((String) model.getPropertyValue("Result Path"));
	stringsPanel.setOrderable(true);
	stringsPanel.setShowValues(false);
	stringsPanel.setTXTStopFiles(stoplist);
	middlePanel.add(stringsPanel);
	stringsPanel.setBorder(new TitledBorder("List"));
	middlePanel.setBorder(new TitledBorder("Strings")); 

	//	middlePanel.add(Box.createVerticalGlue());

	return middlePanel;
    }
			
    private JPanel constructLeftPanel(){
	JPanel leftPanel = new JPanel();
	leftPanel.setPreferredSize(STOPLISTPANELS_PREFERRED_SIZE);
	leftPanel.setLayout(new BoxLayout(leftPanel, BoxLayout.Y_AXIS));

	propertiesPanel = new PropertiesPanel(stoplist);
	leftPanel.add(propertiesPanel);
	return leftPanel;
    }

    private boolean row(){
	boolean row=true;
	if(rowsOrColumns.getText().equals("Columns"))
	    row=false;
	return row;
    }

		
    public void actionPerformed(ActionEvent e) {
	if(e.getSource() instanceof JButton){
	    JButton source = (JButton) e.getSource();
	    IMatrix matrix = model.getIMatrix();
	    if(source == apply){

		//		ProgressFrame pf = new ProgressFrame("Clustering - "+algorithm.getAlgorithmName());
					
		boolean row = row();

		//		stoplist.removeStopIObjects(matrix, row);
		model.getGUIIMatrix().removeStopIObjects(stoplist, row);
		
		
		
					
		view.validateIMatrixBorders();
		view.forceChange();
		view.getOverview().forceChange();

					
		model.setChanged();
		model.notifyObservers();

		//		pf.dispose();
		/*
	    }else if(source == load){
		load();
		stringsPanel.setIVector(stoplist.getStopStringsVector());
		stoplistPanel.repaint();
		repaint();
	    }else if(source == save){
		save();
		*/
	    }else if(source == fromStringsToIO){
		boolean row = row();
		IVector<IObject> newStopIOVector = 
		    stopIObjectsFromStrings();
		iobjectsPanel.setIVector(newStopIOVector);
		stoplist.setStopIObjectsVector(newStopIOVector);
		
		stoplistPanel.repaint();
		repaint();
	    }else if(source == stoplistToIO){
		boolean row = row();
		stoplist.constructStopIObjectsFromStopStrings(matrix, row);
		iobjectsPanel.setIVector(stoplist.getStopIObjectsVector());

		stoplistPanel.repaint();
		repaint();
	    }else if(source == fromIOToStrings){
		stoplist.constructStopStringsFromStopIObjects();
		stringsPanel.setIVector(stoplist.getStopStringsVector());
		stoplistPanel.repaint();
		repaint();
	    }else if(source == cancel){
		setVisible(false);
	    }else if(source == rowsOrColumns){
		rowsOrColumns.press();
		boolean row = row();
		IObjectSet set = matrix.getColumnIObjectSet();
		if(row)
		    set = matrix.getRowIObjectSet();
		iobjectsPanel.setRestrictionGroup(set);

		/*
		if(rowsOrColumns.getText().equals("Columns"))
		    rowsOrColumns.setText("Rows");
		else if(rowsOrColumns.getText().equals("Rows"))
		    rowsOrColumns.setText("Columns");
		*/
		stoplistPanel.repaint();
		repaint();
	    }
	}			
	
    }

    private IVector<IObject> stopIObjectsFromStrings(){
	IVector<IVectorString> stringVec = stringsPanel.getIVector();
	HashSet<String> tempStoplist = new HashSet<String>();
	Vector<IVectorString> vec = stringVec.getUnorderedVector();
	for(int i=0; i<vec.size(); i++){
	    IVectorString ii = vec.get(i);
	    tempStoplist.add(ii.getString());
	}

	IMatrix matrix = model.getIMatrix();

	boolean row = row();
	IObjectSet ioset = matrix.getColumnIObjectSet();
	if(row)
	    ioset = matrix.getRowIObjectSet();
	Vector<IObject> ios = ioset.getOrdered();
	float numberOfIObjects = ioset.numberOfIObjects();

	IVector<IObject> stopIObjects = new IVector<IObject>();
	for(int i=0; i<ios.size(); i++){
	    IObject io = ios.elementAt(i);
	    String s = io.getAmbiguousString();

	    if(tempStoplist.contains(s)){ // stoplist
		stopIObjects.add(io, numberOfIObjects/(i+1));
	    }
	}
	stopIObjects.sortLiteral();

	return stopIObjects;
    }



    /*
    private void load(){
		//		stoplist = new Stoplist("/afs/nada.kth.se/home/f/u1ya3frf/Ordlistor/stoppord/Stoplist.txt");
	//		stoplist = new Stoplist("/afs/nada.kth.se/home/f/u1ya3frf/Ordlistor/english/porter_plus_glasgow_stoplist.txt");
		
	fileChooser.setDialogTitle("Load");
	fileChooser.setSelectedFile(new File(""));
	if (fileChooser.showDialog(getParent(), "Load") == JFileChooser.APPROVE_OPTION) {
	    File file = fileChooser.getSelectedFile();
	    stoplist.readFile(file);
	}
    }


    public void save(){
	fileChooser.setDialogTitle("Save");
	fileChooser.setSelectedFile(new File(""));
	if (fileChooser.showDialog(getParent(), "Save") == JFileChooser.APPROVE_OPTION) {
	    IVectorIO<IVectorIndex> ivio = new IVectorIO<IVectorIndex>();
	    try{
		File file = fileChooser.getSelectedFile();
		if(!file.getCanonicalPath().endsWith(".xml"))
		    file = new File(file.getCanonicalPath()+".xml");
		stoplist.writeFile(file);
	    }catch(IOException ioe){
		showMessage("IOException", "An IOException occured when trying to save.");
	    }
	}
    }
    */



    private void showMessage(String title, String message){
	JOptionPane.showMessageDialog(
				      this, message, title, 
				      JOptionPane.INFORMATION_MESSAGE);

    }




    public void paint(Graphics g){
	super.paint(g);
    }

    private class SmallButton extends JButton{

	public SmallButton(String string){
	    super(string);
	    JTextField dummy = new JTextField(string);
	    JButton dummyButton = new JButton(string);
	    int height = (int) dummy.getPreferredSize().getHeight();
	    int width = (int) dummyButton.getPreferredSize().getWidth();
	    Dimension buttonSize = new Dimension(width, height);
	    setPreferredSize(buttonSize);
	    setMaximumSize(buttonSize);
	}

    }
		
}

