package mro.util.measures;

import mro.util.*;

import infomat.vectorspace.IObjectGroup;

import java.io.*;
import java.util.*; 

/**
 * Averagor.
 *
 * @version          080324
 * @author           Magnus Rosell
 */

public class Averagor{
    //    private Measures measures;
    //    private PropertiesPool pp;

    private XMLFileFilter xmlFF = new XMLFileFilter();

    public void readAndWrite(File dir, String outFileName) 
	throws IOException, MeasureNameConflictException{

	//	System.out.println("in: "+dir.getPath());
	Measures avgs = readAllMeasuresInDirectory(dir);
	//	System.out.println(avgs.getMeasureVector().elementAt(0).getName());
	//	System.out.println("out: "+outFileName);
	avgs.save(outFileName);
    }


    public Measures readAllMeasuresInDirectory(File dir) 
	throws IOException, MeasureNameConflictException{

	Measures avgs = new Measures("Average");

	if(!dir.isDirectory()){
	    System.out.println(dir.getPath());
	    throw new IOException("Averagor: Not a directory!");
	}
	File[] list = dir.listFiles(xmlFF);
	for(int i=0; i<list.length; i++){
	    if(!list[i].isDirectory()){
		//		System.out.println(" - "+list[i].getName());
		Measures measures = Measures.load(list[i]);
		if(measures!=null){
		    //		    System.out.println("*** "+measures.getMeasuresName());
		    //		addMeasures(measures, avgs);
		    recursivelyAddMeasuresForAveraging(measures, avgs);
		}
	    }
	}
	return avgs;
    }


    /*
    private void addMeasures(Measures measures, Measures avgs){
	Vector<Measure> vec = measures.getMeasureVector();
	for(int i=0; i<vec.size(); i++){
	    Measure m = vec.elementAt(i);
	    System.out.println("***** "+m.getName());
	    
	    addMeasure("", m, avgs);
	}
    }


    private void addMeasure(String nameStart, Measure m, Measures avgs){
	if(m instanceof MultiMeasure)
	    return;

	if(m instanceof MeasureGroup){
	    MeasureGroup mg = (MeasureGroup) m;
	    Vector<Measure> vec = mg.getMeasureVector();
	    for(int i=0; i<vec.size(); i++)
		addMeasure(mg.getName(), vec.elementAt(i), avgs);
	    return;
	}

	Measure me = avgs.getMeasure(m.getName());
	if(me==null){
	    try{
		AverageMeasure am = new AverageMeasure(m);
		avgs.addMeasure(am);
	    }catch(MeasureNameConflictException mnce){
		System.out.println("Averagor: MeasureNameConflictException");
	    }
	}else{
	    AverageMeasure am = (AverageMeasure) me;
	    am.addMeasure(m);
	}
    }
    */



    private void recursivelyAddMeasuresForAveraging(Measures measures, 
						    Measures avgs)
	throws MeasureNameConflictException{

	Vector<Measure> vec = measures.getMeasureVector();
	for(int i=0; i<vec.size(); i++){
	    Measure m = vec.elementAt(i);
	    if(m instanceof MeasureGroup){
		MeasureGroup mg = (MeasureGroup) m;
		MeasureGroup avgGroup = (MeasureGroup) avgs;
		recursivelyAddMeasureGroupForAveraging(mg, avgGroup);
	    }
	    else{
		simpleAddingOfMeasureForAveraging(m, avgs);
	    }
	}
    }

    private void recursivelyAddMeasureGroupForAveraging(MeasureGroup mg, 
							MeasureGroup avg)
	throws MeasureNameConflictException{

	MeasureGroup newGroup = avg.getMeasureGroup(mg.getName());
	if(newGroup==null){
	    newGroup = new MeasureGroup(mg.getName());
	    avg.addMeasure(newGroup);
	}

	Vector<Measure> measureVector = mg.getMeasureVector();
	for(int i=0; i<measureVector.size(); i++){
	    Measure m = measureVector.elementAt(i);
	    if(m instanceof MeasureGroup)
		recursivelyAddMeasureGroupForAveraging((MeasureGroup) m, newGroup);
	    else
		simpleAddingOfMeasureForAveraging(m, newGroup);
	}
    }

    private void simpleAddingOfMeasureForAveraging(Measure m, 
						   MeasureGroup avg)
	throws MeasureNameConflictException{
	Measure me = avg.getMeasure(m.getName());
	if(me==null){
	    AverageMeasure am = new AverageMeasure(m);
	    avg.addMeasure(am);
	}else{
	    AverageMeasure am = (AverageMeasure) me;
	    am.addMeasure(m);
	}
    }


    private class XMLFileFilter implements FileFilter{

	public boolean accept(File pathname){
	    if(pathname.getName().toLowerCase().endsWith(".xml"))
		return true;
	    return false;
	}
    }


}



